!OPTION! -Nfusion
      SUBROUTINE CH_PRODLOSS(PTIME,PCONC,PPROD,PLOSS,KMI,KVECNPT,KEQ)
      USE PARKIND1, ONLY : JPRB
      USE YOMHOOK , ONLY : LHOOK, DR_HOOK
!!    ###################################################
!! This code has been created automatically by preprocessor m10,
!! version: 9.7, copyright 1995-1999 by Meteo France/Universite Paul Sabatier.
!! Please report all bugs to K. Suhre (Lab. d'Aerologie UPS/CNRS).
!!
!!*** *MODD_CH_PRODLOSS*
!!
!!    PURPOSE
!!    -------
!       calculation of production and loss terms for diagnostics
!!
!!**  METHOD
!!    ------
!!      The terms of temporal derivative of the chemical species,
!!    written as:
!!    d/dt PCONC = CH_FCN = PPROD - PCONC*PLOSS,
!!    are calculated for diagnostic purposes.
!!
!!    REFERENCE
!!    ---------
!!    none
!!
!!    AUTHOR
!!    ------
!!    Karsten Suhre (LA)
!!
!!    MODIFICATIONS
!!    -------------
!!    Original 26/07/96
!!    Modified 05/05/98: Vectorization (Vincent Crassier & KS)
!!    Modified 31/10/03: New interface for better MesoNH compilation (D. Gazen)
!!
!!----------------------------------------------------------------------
!!
!!    EXTERNAL
!!    --------
!!    none
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
USE MODD_CH_M9_SCHEME
!!
!!    EXPLICIT ARGUMENTS
!!    ------------------
IMPLICIT NONE
REAL,    INTENT(IN)                      :: PTIME
INTEGER, INTENT(IN)                      :: KVECNPT
INTEGER, INTENT(IN)                      :: KEQ
REAL,    INTENT(IN),  DIMENSION(KVECNPT,KEQ)     :: PCONC
REAL,    INTENT(OUT), DIMENSION(KVECNPT,KEQ)     :: PPROD, PLOSS
INTEGER, INTENT(IN)                          :: KMI
TYPE(CCSTYPE), POINTER         :: TPK
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('CH_PRODLOSS',0,ZHOOK_HANDLE)
TPK=>TACCS(KMI)
! /BEGIN_CODE/

 TPK%O1D(:)=(TPK%K002(:)*PCONC(:,JP_O3))/(TPK%K020(:)*TPK%N2(:)+TPK%K021(:)*TPK%O2(:)+&
        &TPK%K022(:)*TPK%H2O(:))

 TPK%O3P(:)=(TPK%K001(:)*PCONC(:,JP_NO2)+TPK%K003(:)*PCONC(:,JP_O3)+TPK%K008(:)*PCONC(:,JP_NO3)+&
   &TPK%K020(:)*TPK%O1D(:)*TPK%N2(:)+TPK%K021(:)*TPK%O1D(:)*TPK%O2(:)+&
   &0.09000*TPK%K077(:)*PCONC(:,JP_BIO)*PCONC(:,JP_O3))/&
   &(TPK%K018(:)*TPK%O2(:)+TPK%K019(:)*PCONC(:,JP_O3)+TPK%K029(:)*PCONC(:,JP_NO)+&
   &TPK%K030(:)*PCONC(:,JP_NO2)+TPK%K031(:)*PCONC(:,JP_NO2)+TPK%K053(:)*PCONC(:,JP_BIO)+&
   &TPK%K054(:)*PCONC(:,JP_CARBO))


! /END_CODE/
CALL SUB0
CALL SUB1
CALL SUB2
CALL SUB3

IF (LHOOK) CALL DR_HOOK('CH_PRODLOSS',1,ZHOOK_HANDLE)
CONTAINS

!OPTION! -Nfusion
SUBROUTINE SUB0
!
!Indices 1 a 10
!
!
!PPROD(O3) = +K018*<O3P>*<O2>+0.17307*K099*<CARBOP>*<HO2>
 REAL(KIND=JPRB) :: ZHOOK_HANDLE
 IF (LHOOK) CALL DR_HOOK('CH_PRODLOSS:SUB0',0,ZHOOK_HANDLE)
 PPROD(:,1) = +TPK%K018(:)*TPK%O3P(:)*TPK%O2(:)+0.17307*TPK%K099(:)*PCONC(:,38)&
&*PCONC(:,15)
!PLOSS(O3) = +K002+K003+K019*<O3P>+K023*<OH>+K024*<HO2>+K042*<NO>+K043*<NO2>+K0
!76*<ALKE>+K077*<BIO>+K078*<CARBO>+K079*<PAN>+K084*<ADD>
 PLOSS(:,1) = +TPK%K002(:)+TPK%K003(:)+TPK%K019(:)*TPK%O3P(:)+TPK%K023(:)*PCONC&
&(:,14)+TPK%K024(:)*PCONC(:,15)+TPK%K042(:)*PCONC(:,3)+TPK%K043(:)*PCONC(:,4)+T&
&PK%K076(:)*PCONC(:,19)+TPK%K077(:)*PCONC(:,20)+TPK%K078(:)*PCONC(:,25)+TPK%K07&
&9(:)*PCONC(:,27)+TPK%K084(:)*PCONC(:,36)
!
!PPROD(H2O2) = +K027*<HO2>*<HO2>+K028*<HO2>*<HO2>*<H2O>+0.01833*K076*<ALKE>*<O3
!>+0.00100*K077*<BIO>*<O3>
 PPROD(:,2) = +TPK%K027(:)*PCONC(:,15)*PCONC(:,15)+TPK%K028(:)*PCONC(:,15)*PCON&
&C(:,15)*TPK%H2O(:)+0.01833*TPK%K076(:)*PCONC(:,19)*PCONC(:,1)+0.00100*TPK%K077&
&(:)*PCONC(:,20)*PCONC(:,1)
!PLOSS(H2O2) = +K009+K026*<OH>
 PLOSS(:,2) = +TPK%K009(:)+TPK%K026(:)*PCONC(:,14)
!
!PPROD(NO) = +K001*<NO2>+K004*<HONO>+K007*<NO3>+K030*<O3P>*<NO2>+K046*<NO3>*<NO
!2>
 PPROD(:,3) = +TPK%K001(:)*PCONC(:,4)+TPK%K004(:)*PCONC(:,7)+TPK%K007(:)*PCONC(&
&:,5)+TPK%K030(:)*TPK%O3P(:)*PCONC(:,4)+TPK%K046(:)*PCONC(:,5)*PCONC(:,4)
!PLOSS(NO) = +K029*<O3P>+K032*<OH>+K035*<HO2>+K042*<O3>+K044*<NO>*<O2>+K044*<NO
!>*<O2>+K045*<NO3>+K087*<MO2>+K088*<ALKAP>+K089*<ALKEP>+K090*<BIOP>+K091*<AROP>
!+K092*<CARBOP>+K093*<OLN>+K127*<XO2>
 PLOSS(:,3) = +TPK%K029(:)*TPK%O3P(:)+TPK%K032(:)*PCONC(:,14)+TPK%K035(:)*PCONC&
&(:,15)+TPK%K042(:)*PCONC(:,1)+TPK%K044(:)*PCONC(:,3)*TPK%O2(:)+TPK%K044(:)*PCO&
&NC(:,3)*TPK%O2(:)+TPK%K045(:)*PCONC(:,5)+TPK%K087(:)*PCONC(:,31)+TPK%K088(:)*P&
&CONC(:,32)+TPK%K089(:)*PCONC(:,33)+TPK%K090(:)*PCONC(:,34)+TPK%K091(:)*PCONC(:&
&,37)+TPK%K092(:)*PCONC(:,38)+TPK%K093(:)*PCONC(:,39)+TPK%K127(:)*PCONC(:,40)
!

!PPROD(NO2) = +K005*<HNO3>+0.65*K006*<HNO4>+K008*<NO3>+K017*<ONIT>+K029*<O3P>*<
!NO>+K034*<OH>*<NO3>+K035*<HO2>*<NO>+K037*<HNO4>+0.7*K038*<HO2>*<NO3>+K039*<OH>
!*<HONO>+K041*<OH>*<HNO4>+K042*<O3>*<NO>+K044*<NO>*<NO>*<O2>+K044*<NO>*<NO>*<O2
!>+K045*<NO3>*<NO>+K045*<NO3>*<NO>+K046*<NO3>*<NO2>+K048*<N2O5>+K049*<NO3>*<NO3
!>+K049*<NO3>*<NO3>+K068*<ONIT>*<OH>+0.10530*K071*<CARBO>*<NO3>+0.40*K075*<PAN>
!*<NO3>+0.70*K079*<PAN>*<O3>+K086*<PAN>+K087*<MO2>*<NO>+0.91541*K088*<ALKAP>*<N
!O>+K089*<ALKEP>*<NO>+0.84700*K090*<BIOP>*<NO>+0.95115*K091*<AROP>*<NO>+K092*<C
!ARBOP>*<NO>+1.81599*K093*<OLN>*<NO>+0.32440*K107*<OLN>*<MO2>+0.00000*K113*<OLN
!>*<CARBOP>+0.00000*K115*<OLN>*<OLN>+K116*<MO2>*<NO3>+K117*<ALKAP>*<NO3>+K118*<
!ALKEP>*<NO3>+K119*<BIOP>*<NO3>+K120*<AROP>*<NO3>+K121*<CARBOP>*<NO3>+1.74072*K
!122*<OLN>*<NO3>+K127*<XO2>*<NO>+K128*<XO2>*<NO3>
 PPROD(:,4) = +TPK%K005(:)*PCONC(:,8)+0.65*TPK%K006(:)*PCONC(:,9)+TPK%K008(:)*P&
&CONC(:,5)+TPK%K017(:)*PCONC(:,26)+TPK%K029(:)*TPK%O3P(:)*PCONC(:,3)+TPK%K034(:&
&)*PCONC(:,14)*PCONC(:,5)+TPK%K035(:)*PCONC(:,15)*PCONC(:,3)+TPK%K037(:)*PCONC(&
&:,9)+0.7*TPK%K038(:)*PCONC(:,15)*PCONC(:,5)+TPK%K039(:)*PCONC(:,14)*PCONC(:,7)&
&+TPK%K041(:)*PCONC(:,14)*PCONC(:,9)+TPK%K042(:)*PCONC(:,1)*PCONC(:,3)+TPK%K044&
&(:)*PCONC(:,3)*PCONC(:,3)*TPK%O2(:)+TPK%K044(:)*PCONC(:,3)*PCONC(:,3)*TPK%O2(:&
&)+TPK%K045(:)*PCONC(:,5)*PCONC(:,3)+TPK%K045(:)*PCONC(:,5)*PCONC(:,3)+TPK%K046&
&(:)*PCONC(:,5)*PCONC(:,4)+TPK%K048(:)*PCONC(:,6)+TPK%K049(:)*PCONC(:,5)*PCONC(&
&:,5)+TPK%K049(:)*PCONC(:,5)*PCONC(:,5)+TPK%K068(:)*PCONC(:,26)*PCONC(:,14)+0.1&
&0530*TPK%K071(:)*PCONC(:,25)*PCONC(:,5)+0.40*TPK%K075(:)*PCONC(:,27)*PCONC(:,5&
&)+0.70*TPK%K079(:)*PCONC(:,27)*PCONC(:,1)+TPK%K086(:)*PCONC(:,27)+TPK%K087(:)*&
&PCONC(:,31)*PCONC(:,3)+0.91541*TPK%K088(:)*PCONC(:,32)*PCONC(:,3)+TPK%K089(:)*&
&PCONC(:,33)*PCONC(:,3)+0.84700*TPK%K090(:)*PCONC(:,34)*PCONC(:,3)+0.95115*TPK%&
&K091(:)*PCONC(:,37)*PCONC(:,3)+TPK%K092(:)*PCONC(:,38)*PCONC(:,3)+1.81599*TPK%&
&K093(:)*PCONC(:,39)*PCONC(:,3)+0.32440*TPK%K107(:)*PCONC(:,39)*PCONC(:,31)+0.0&
&0000*TPK%K113(:)*PCONC(:,39)*PCONC(:,38)+0.00000*TPK%K115(:)*PCONC(:,39)*PCONC&
&(:,39)+TPK%K116(:)*PCONC(:,31)*PCONC(:,5)+TPK%K117(:)*PCONC(:,32)*PCONC(:,5)+T&
&PK%K118(:)*PCONC(:,33)*PCONC(:,5)+TPK%K119(:)*PCONC(:,34)*PCONC(:,5)+TPK%K120(&
&:)*PCONC(:,37)*PCONC(:,5)+TPK%K121(:)*PCONC(:,38)*PCONC(:,5)+1.74072*TPK%K122(&
&:)*PCONC(:,39)*PCONC(:,5)+TPK%K127(:)*PCONC(:,40)*PCONC(:,3)+TPK%K128(:)*PCONC&
&(:,40)*PCONC(:,5)
!PLOSS(NO2) = +K001+K030*<O3P>+K031*<O3P>+K033*<OH>+K036*<HO2>+K043*<O3>+K046*<
!NO3>+K047*<NO3>+K080*<PHO>+K082*<ADD>+K085*<CARBOP>
 PLOSS(:,4) = +TPK%K001(:)+TPK%K030(:)*TPK%O3P(:)+TPK%K031(:)*TPK%O3P(:)+TPK%K0&
&33(:)*PCONC(:,14)+TPK%K036(:)*PCONC(:,15)+TPK%K043(:)*PCONC(:,1)+TPK%K046(:)*P&
&CONC(:,5)+TPK%K047(:)*PCONC(:,5)+TPK%K080(:)*PCONC(:,35)+TPK%K082(:)*PCONC(:,3&
&6)+TPK%K085(:)*PCONC(:,38)

!
!PPROD(NO3) = +0.35*K006*<HNO4>+K031*<O3P>*<NO2>+K040*<OH>*<HNO3>+K043*<O3>*<NO
!2>+K048*<N2O5>+0.71893*K067*<PAN>*<OH>+0.60*K075*<PAN>*<NO3>
 PPROD(:,5) = +0.35*TPK%K006(:)*PCONC(:,9)+TPK%K031(:)*TPK%O3P(:)*PCONC(:,4)+TP&
&K%K040(:)*PCONC(:,14)*PCONC(:,8)+TPK%K043(:)*PCONC(:,1)*PCONC(:,4)+TPK%K048(:)&
&*PCONC(:,6)+0.71893*TPK%K067(:)*PCONC(:,27)*PCONC(:,14)+0.60*TPK%K075(:)*PCONC&
&(:,27)*PCONC(:,5)
!PLOSS(NO3) = +K007+K008+K034*<OH>+K038*<HO2>+K045*<NO>+K046*<NO2>+K047*<NO2>+K
!049*<NO3>+K049*<NO3>+K069*<HCHO>+K070*<ALD>+K071*<CARBO>+K072*<ARO>+K073*<ALKE
!>+K074*<BIO>+K075*<PAN>+K116*<MO2>+K117*<ALKAP>+K118*<ALKEP>+K119*<BIOP>+K120*
!<AROP>+K121*<CARBOP>+K122*<OLN>+K128*<XO2>
 PLOSS(:,5) = +TPK%K007(:)+TPK%K008(:)+TPK%K034(:)*PCONC(:,14)+TPK%K038(:)*PCON&
&C(:,15)+TPK%K045(:)*PCONC(:,3)+TPK%K046(:)*PCONC(:,4)+TPK%K047(:)*PCONC(:,4)+T&
&PK%K049(:)*PCONC(:,5)+TPK%K049(:)*PCONC(:,5)+TPK%K069(:)*PCONC(:,22)+TPK%K070(&
&:)*PCONC(:,23)+TPK%K071(:)*PCONC(:,25)+TPK%K072(:)*PCONC(:,21)+TPK%K073(:)*PCO&
&NC(:,19)+TPK%K074(:)*PCONC(:,20)+TPK%K075(:)*PCONC(:,27)+TPK%K116(:)*PCONC(:,3&
&1)+TPK%K117(:)*PCONC(:,32)+TPK%K118(:)*PCONC(:,33)+TPK%K119(:)*PCONC(:,34)+TPK&
&%K120(:)*PCONC(:,37)+TPK%K121(:)*PCONC(:,38)+TPK%K122(:)*PCONC(:,39)+TPK%K128(&
&:)*PCONC(:,40)
!
!PPROD(N2O5) = +K047*<NO3>*<NO2>
 PPROD(:,6) = +TPK%K047(:)*PCONC(:,5)*PCONC(:,4)
!PLOSS(N2O5) = +K048
 PLOSS(:,6) = +TPK%K048(:)
!
!PPROD(HONO) = +K032*<OH>*<NO>+K082*<ADD>*<NO2>
 PPROD(:,7) = +TPK%K032(:)*PCONC(:,14)*PCONC(:,3)+TPK%K082(:)*PCONC(:,36)*PCONC&
&(:,4)
!PLOSS(HONO) = +K004+K039*<OH>
 PLOSS(:,7) = +TPK%K004(:)+TPK%K039(:)*PCONC(:,14)
!
!PPROD(HNO3) = +K033*<OH>*<NO2>+0.3*K038*<HO2>*<NO3>+K069*<HCHO>*<NO3>+K070*<AL
!D>*<NO3>+0.91567*K071*<CARBO>*<NO3>+K072*<ARO>*<NO3>
 PPROD(:,8) = +TPK%K033(:)*PCONC(:,14)*PCONC(:,4)+0.3*TPK%K038(:)*PCONC(:,15)*P&
&CONC(:,5)+TPK%K069(:)*PCONC(:,22)*PCONC(:,5)+TPK%K070(:)*PCONC(:,23)*PCONC(:,5&
&)+0.91567*TPK%K071(:)*PCONC(:,25)*PCONC(:,5)+TPK%K072(:)*PCONC(:,21)*PCONC(:,5&
&)
!PLOSS(HNO3) = +K005+K040*<OH>
 PLOSS(:,8) = +TPK%K005(:)+TPK%K040(:)*PCONC(:,14)
!
!PPROD(HNO4) = +K036*<HO2>*<NO2>
 PPROD(:,9) = +TPK%K036(:)*PCONC(:,15)*PCONC(:,4)
!PLOSS(HNO4) = +K006+K037+K041*<OH>
 PLOSS(:,9) = +TPK%K006(:)+TPK%K037(:)+TPK%K041(:)*PCONC(:,14)
!
!PPROD(NH3) = 0.0
 PPROD(:,10) = 0.0
!PLOSS(NH3) = 0.0
 PLOSS(:,10) = 0.0
!
IF (LHOOK) CALL DR_HOOK('CH_PRODLOSS:SUB0',1,ZHOOK_HANDLE)
RETURN
END SUBROUTINE SUB0
!
!OPTION! -Nfusion
SUBROUTINE SUB1
!
!Indices 11 a 20
!
!
!PPROD(SO2) = 0.0
 REAL(KIND=JPRB) :: ZHOOK_HANDLE
 IF (LHOOK) CALL DR_HOOK('CH_PRODLOSS:SUB1',0,ZHOOK_HANDLE)
 PPROD(:,11) = 0.0
!PLOSS(SO2) = +K051*<OH>
 PLOSS(:,11) = +TPK%K051(:)*PCONC(:,14)
!
!PPROD(SULF) = +K051*<OH>*<SO2>
 PPROD(:,12) = +TPK%K051(:)*PCONC(:,14)*PCONC(:,11)
!PLOSS(SULF) = 0.0
 PLOSS(:,12) = 0.0
!
!PPROD(CO) = +K010*<HCHO>+K011*<HCHO>+K012*<ALD>+0.91924*K016*<CARBO>+0.01*K053
!*<BIO>*<O3P>+0.00878*K057*<ALKA>*<OH>+K061*<HCHO>*<OH>+1.01732*K064*<CARBO>*<O
!H>+K069*<HCHO>*<NO3>+1.33723*K071*<CARBO>*<NO3>+0.35120*K076*<ALKE>*<O3>+0.360
!00*K077*<BIO>*<O3>+0.64728*K078*<CARBO>*<O3>+0.13*K079*<PAN>*<O3>
 PPROD(:,13) = +TPK%K010(:)*PCONC(:,22)+TPK%K011(:)*PCONC(:,22)+TPK%K012(:)*PCO&
&NC(:,23)+0.91924*TPK%K016(:)*PCONC(:,25)+0.01*TPK%K053(:)*PCONC(:,20)*TPK%O3P(&
&:)+0.00878*TPK%K057(:)*PCONC(:,18)*PCONC(:,14)+TPK%K061(:)*PCONC(:,22)*PCONC(:&
&,14)+1.01732*TPK%K064(:)*PCONC(:,25)*PCONC(:,14)+TPK%K069(:)*PCONC(:,22)*PCONC&
&(:,5)+1.33723*TPK%K071(:)*PCONC(:,25)*PCONC(:,5)+0.35120*TPK%K076(:)*PCONC(:,1&
&9)*PCONC(:,1)+0.36000*TPK%K077(:)*PCONC(:,20)*PCONC(:,1)+0.64728*TPK%K078(:)*P&
&CONC(:,25)*PCONC(:,1)+0.13*TPK%K079(:)*PCONC(:,27)*PCONC(:,1)
!PLOSS(CO) = +K052*<OH>
 PLOSS(:,13) = +TPK%K052(:)*PCONC(:,14)
!
!PPROD(OH) = +K004*<HONO>+K005*<HNO3>+0.35*K006*<HNO4>+K009*<H2O2>+K009*<H2O2>+
!K013*<OP1>+K014*<OP2>+K022*<O1D>*<H2O>+K022*<O1D>*<H2O>+K024*<O3>*<HO2>+K035*<
!HO2>*<NO>+0.7*K038*<HO2>*<NO3>+0.02*K053*<BIO>*<O3P>+0.00878*K057*<ALKA>*<OH>+
!0.35*K065*<OP1>*<OH>+0.44925*K066*<OP2>*<OH>+0.39435*K076*<ALKE>*<O3>+0.28000*
!K077*<BIO>*<O3>+0.20595*K078*<CARBO>*<O3>+0.036*K079*<PAN>*<O3>+K084*<ADD>*<O3
!>
 PPROD(:,14) = +TPK%K004(:)*PCONC(:,7)+TPK%K005(:)*PCONC(:,8)+0.35*TPK%K006(:)*&
&PCONC(:,9)+TPK%K009(:)*PCONC(:,2)+TPK%K009(:)*PCONC(:,2)+TPK%K013(:)*PCONC(:,2&
&8)+TPK%K014(:)*PCONC(:,29)+TPK%K022(:)*TPK%O1D(:)*TPK%H2O(:)+TPK%K022(:)*TPK%O&
&1D(:)*TPK%H2O(:)+TPK%K024(:)*PCONC(:,1)*PCONC(:,15)+TPK%K035(:)*PCONC(:,15)*PC&
&ONC(:,3)+0.7*TPK%K038(:)*PCONC(:,15)*PCONC(:,5)+0.02*TPK%K053(:)*PCONC(:,20)*T&
&PK%O3P(:)+0.00878*TPK%K057(:)*PCONC(:,18)*PCONC(:,14)+0.35*TPK%K065(:)*PCONC(:&
&,28)*PCONC(:,14)+0.44925*TPK%K066(:)*PCONC(:,29)*PCONC(:,14)+0.39435*TPK%K076(&
&:)*PCONC(:,19)*PCONC(:,1)+0.28000*TPK%K077(:)*PCONC(:,20)*PCONC(:,1)+0.20595*T&
&PK%K078(:)*PCONC(:,25)*PCONC(:,1)+0.036*TPK%K079(:)*PCONC(:,27)*PCONC(:,1)+TPK&
&%K084(:)*PCONC(:,36)*PCONC(:,1)
!PLOSS(OH) = +K023*<O3>+K025*<HO2>+K026*<H2O2>+K032*<NO>+K033*<NO2>+K034*<NO3>+
!K039*<HONO>+K040*<HNO3>+K041*<HNO4>+K050*<H2>+K051*<SO2>+K052*<CO>+K055*<CH4>+
!K056*<ETH>+K057*<ALKA>+K058*<ALKE>+K059*<BIO>+K060*<ARO>+K061*<HCHO>+K062*<ALD
!>+K063*<KET>+K064*<CARBO>+K065*<OP1>+K066*<OP2>+K067*<PAN>+K068*<ONIT>
 PLOSS(:,14) = +TPK%K023(:)*PCONC(:,1)+TPK%K025(:)*PCONC(:,15)+TPK%K026(:)*PCON&
&C(:,2)+TPK%K032(:)*PCONC(:,3)+TPK%K033(:)*PCONC(:,4)+TPK%K034(:)*PCONC(:,5)+TP&
&K%K039(:)*PCONC(:,7)+TPK%K040(:)*PCONC(:,8)+TPK%K041(:)*PCONC(:,9)+TPK%K050(:)&
&*TPK%H2(:)+TPK%K051(:)*PCONC(:,11)+TPK%K052(:)*PCONC(:,13)+TPK%K055(:)*PCONC(:&
&,16)+TPK%K056(:)*PCONC(:,17)+TPK%K057(:)*PCONC(:,18)+TPK%K058(:)*PCONC(:,19)+T&
&PK%K059(:)*PCONC(:,20)+TPK%K060(:)*PCONC(:,21)+TPK%K061(:)*PCONC(:,22)+TPK%K06&
&2(:)*PCONC(:,23)+TPK%K063(:)*PCONC(:,24)+TPK%K064(:)*PCONC(:,25)+TPK%K065(:)*P&
&CONC(:,28)+TPK%K066(:)*PCONC(:,29)+TPK%K067(:)*PCONC(:,27)+TPK%K068(:)*PCONC(:&
&,26)
!
!PPROD(HO2) = +0.65*K006*<HNO4>+K011*<HCHO>+K011*<HCHO>+K012*<ALD>+K013*<OP1>+0
!.96205*K014*<OP2>+0.75830*K016*<CARBO>+K017*<ONIT>+K023*<O3>*<OH>+K026*<H2O2>*
!<OH>+K034*<OH>*<NO3>+K037*<HNO4>+K050*<OH>*<H2>+K051*<OH>*<SO2>+K052*<CO>*<OH>
!+0.28*K053*<BIO>*<O3P>+0.12793*K057*<ALKA>*<OH>+0.10318*K060*<ARO>*<OH>+K061*<
!HCHO>*<OH>+0.51208*K064*<CARBO>*<OH>+0.02915*K066*<OP2>*<OH>+0.28107*K067*<PAN
!>*<OH>+K069*<HCHO>*<NO3>+0.63217*K071*<CARBO>*<NO3>+0.23451*K076*<ALKE>*<O3>+0
!.30000*K077*<BIO>*<O3>+0.28441*K078*<CARBO>*<O3>+0.08*K079*<PAN>*<O3>+0.02*K08
!3*<ADD>*<O2>+K087*<MO2>*<NO>+0.74265*K088*<ALKAP>*<NO>+K089*<ALKEP>*<NO>+0.847
!00*K090*<BIOP>*<NO>+0.95115*K091*<AROP>*<NO>+0.12334*K092*<CARBOP>*<NO>+0.1840
!1*K093*<OLN>*<NO>+0.66*K101*<MO2>*<MO2>+0.98383*K102*<ALKAP>*<MO2>+K103*<ALKEP
!>*<MO2>+1.00000*K104*<BIOP>*<MO2>+1.02767*K105*<AROP>*<MO2>+0.82998*K106*<CARB
!OP>*<MO2>+0.67560*K107*<OLN>*<MO2>+0.48079*K108*<ALKAP>*<CARBOP>+0.50078*K109*
!<ALKEP>*<CARBOP>+0.50600*K110*<BIOP>*<CARBOP>+K111*<AROP>*<CARBOP>+0.07566*K11
!2*<CARBOP>*<CARBOP>+0.17599*K113*<OLN>*<CARBOP>+K114*<OLN>*<OLN>+0.00000*K115*
!<OLN>*<OLN>+K116*<MO2>*<NO3>+0.81290*K117*<ALKAP>*<NO3>+K118*<ALKEP>*<NO3>+K11
!9*<BIOP>*<NO3>+K120*<AROP>*<NO3>+0.04915*K121*<CARBOP>*<NO3>+0.25928*K122*<OLN
!>*<NO3>+K124*<XO2>*<MO2>
 PPROD(:,15) = +0.65*TPK%K006(:)*PCONC(:,9)+TPK%K011(:)*PCONC(:,22)+TPK%K011(:)&
&*PCONC(:,22)+TPK%K012(:)*PCONC(:,23)+TPK%K013(:)*PCONC(:,28)+0.96205*TPK%K014(&
&:)*PCONC(:,29)+0.75830*TPK%K016(:)*PCONC(:,25)+TPK%K017(:)*PCONC(:,26)+TPK%K02&
&3(:)*PCONC(:,1)*PCONC(:,14)+TPK%K026(:)*PCONC(:,2)*PCONC(:,14)+TPK%K034(:)*PCO&
&NC(:,14)*PCONC(:,5)+TPK%K037(:)*PCONC(:,9)+TPK%K050(:)*PCONC(:,14)*TPK%H2(:)+T&
&PK%K051(:)*PCONC(:,14)*PCONC(:,11)+TPK%K052(:)*PCONC(:,13)*PCONC(:,14)+0.28*TP&
&K%K053(:)*PCONC(:,20)*TPK%O3P(:)+0.12793*TPK%K057(:)*PCONC(:,18)*PCONC(:,14)+0&
&.10318*TPK%K060(:)*PCONC(:,21)*PCONC(:,14)+TPK%K061(:)*PCONC(:,22)*PCONC(:,14)&
&+0.51208*TPK%K064(:)*PCONC(:,25)*PCONC(:,14)+0.02915*TPK%K066(:)*PCONC(:,29)*P&
&CONC(:,14)+0.28107*TPK%K067(:)*PCONC(:,27)*PCONC(:,14)+TPK%K069(:)*PCONC(:,22)&
&*PCONC(:,5)+0.63217*TPK%K071(:)*PCONC(:,25)*PCONC(:,5)+0.23451*TPK%K076(:)*PCO&
&NC(:,19)*PCONC(:,1)+0.30000*TPK%K077(:)*PCONC(:,20)*PCONC(:,1)+0.28441*TPK%K07&
&8(:)*PCONC(:,25)*PCONC(:,1)+0.08*TPK%K079(:)*PCONC(:,27)*PCONC(:,1)+0.02*TPK%K&
&083(:)*PCONC(:,36)*TPK%O2(:)+TPK%K087(:)*PCONC(:,31)*PCONC(:,3)+0.74265*TPK%K0&
&88(:)*PCONC(:,32)*PCONC(:,3)+TPK%K089(:)*PCONC(:,33)*PCONC(:,3)+0.84700*TPK%K0&
&90(:)*PCONC(:,34)*PCONC(:,3)+0.95115*TPK%K091(:)*PCONC(:,37)*PCONC(:,3)+0.1233&
&4*TPK%K092(:)*PCONC(:,38)*PCONC(:,3)+0.18401*TPK%K093(:)*PCONC(:,39)*PCONC(:,3&
&)+0.66*TPK%K101(:)*PCONC(:,31)*PCONC(:,31)+0.98383*TPK%K102(:)*PCONC(:,32)*PCO&
&NC(:,31)+TPK%K103(:)*PCONC(:,33)*PCONC(:,31)+1.00000*TPK%K104(:)*PCONC(:,34)*P&
&CONC(:,31)+1.02767*TPK%K105(:)*PCONC(:,37)*PCONC(:,31)+0.82998*TPK%K106(:)*PCO&
&NC(:,38)*PCONC(:,31)+0.67560*TPK%K107(:)*PCONC(:,39)*PCONC(:,31)+0.48079*TPK%K&
&108(:)*PCONC(:,32)*PCONC(:,38)+0.50078*TPK%K109(:)*PCONC(:,33)*PCONC(:,38)+0.5&
&0600*TPK%K110(:)*PCONC(:,34)*PCONC(:,38)+TPK%K111(:)*PCONC(:,37)*PCONC(:,38)+0&
&.07566*TPK%K112(:)*PCONC(:,38)*PCONC(:,38)+0.17599*TPK%K113(:)*PCONC(:,39)*PCO&
&NC(:,38)+TPK%K114(:)*PCONC(:,39)*PCONC(:,39)+0.00000*TPK%K115(:)*PCONC(:,39)*P&
&CONC(:,39)+TPK%K116(:)*PCONC(:,31)*PCONC(:,5)+0.81290*TPK%K117(:)*PCONC(:,32)*&
&PCONC(:,5)+TPK%K118(:)*PCONC(:,33)*PCONC(:,5)+TPK%K119(:)*PCONC(:,34)*PCONC(:,&
&5)+TPK%K120(:)*PCONC(:,37)*PCONC(:,5)+0.04915*TPK%K121(:)*PCONC(:,38)*PCONC(:,&
&5)+0.25928*TPK%K122(:)*PCONC(:,39)*PCONC(:,5)+TPK%K124(:)*PCONC(:,40)*PCONC(:,&
&31)
!PLOSS(HO2) = +K024*<O3>+K025*<OH>+K027*<HO2>+K027*<HO2>+K028*<HO2>*<H2O>+K028*
!<HO2>*<H2O>+K035*<NO>+K036*<NO2>+K038*<NO3>+K081*<PHO>+K094*<MO2>+K095*<ALKAP>
!+K096*<ALKEP>+K097*<BIOP>+K098*<AROP>+K099*<CARBOP>+K100*<OLN>+K123*<XO2>
 PLOSS(:,15) = +TPK%K024(:)*PCONC(:,1)+TPK%K025(:)*PCONC(:,14)+TPK%K027(:)*PCON&
&C(:,15)+TPK%K027(:)*PCONC(:,15)+TPK%K028(:)*PCONC(:,15)*TPK%H2O(:)+TPK%K028(:)&
&*PCONC(:,15)*TPK%H2O(:)+TPK%K035(:)*PCONC(:,3)+TPK%K036(:)*PCONC(:,4)+TPK%K038&
&(:)*PCONC(:,5)+TPK%K081(:)*PCONC(:,35)+TPK%K094(:)*PCONC(:,31)+TPK%K095(:)*PCO&
&NC(:,32)+TPK%K096(:)*PCONC(:,33)+TPK%K097(:)*PCONC(:,34)+TPK%K098(:)*PCONC(:,3&
&7)+TPK%K099(:)*PCONC(:,38)+TPK%K100(:)*PCONC(:,39)+TPK%K123(:)*PCONC(:,40)
!
!PPROD(CH4) = +0.04300*K076*<ALKE>*<O3>
 PPROD(:,16) = +0.04300*TPK%K076(:)*PCONC(:,19)*PCONC(:,1)
!PLOSS(CH4) = +K055*<OH>
 PLOSS(:,16) = +TPK%K055(:)*PCONC(:,14)
!
!PPROD(ETH) = +0.03196*K076*<ALKE>*<O3>
 PPROD(:,17) = +0.03196*TPK%K076(:)*PCONC(:,19)*PCONC(:,1)
!PLOSS(ETH) = +K056*<OH>
 PLOSS(:,17) = +TPK%K056(:)*PCONC(:,14)
!
!PPROD(ALKA) = 0.0
 PPROD(:,18) = 0.0
!PLOSS(ALKA) = +K057*<OH>
 PLOSS(:,18) = +TPK%K057(:)*PCONC(:,14)
!
!PPROD(ALKE) = +0.91868*K053*<BIO>*<O3P>+0.00000*K076*<ALKE>*<O3>+0.37388*K077*
!<BIO>*<O3>+0.37815*K090*<BIOP>*<NO>+0.48074*K104*<BIOP>*<MO2>+0.24463*K110*<BI
!OP>*<CARBOP>+0.42729*K119*<BIOP>*<NO3>
 PPROD(:,19) = +0.91868*TPK%K053(:)*PCONC(:,20)*TPK%O3P(:)+0.00000*TPK%K076(:)*&
&PCONC(:,19)*PCONC(:,1)+0.37388*TPK%K077(:)*PCONC(:,20)*PCONC(:,1)+0.37815*TPK%&
&K090(:)*PCONC(:,34)*PCONC(:,3)+0.48074*TPK%K104(:)*PCONC(:,34)*PCONC(:,31)+0.2&
&4463*TPK%K110(:)*PCONC(:,34)*PCONC(:,38)+0.42729*TPK%K119(:)*PCONC(:,34)*PCONC&
&(:,5)
!PLOSS(ALKE) = +K058*<OH>+K073*<NO3>+K076*<O3>
 PLOSS(:,19) = +TPK%K058(:)*PCONC(:,14)+TPK%K073(:)*PCONC(:,5)+TPK%K076(:)*PCON&
&C(:,1)
!
!PPROD(BIO) = 0.0
 PPROD(:,20) = 0.0
!PLOSS(BIO) = +K053*<O3P>+K059*<OH>+K074*<NO3>+K077*<O3>
 PLOSS(:,20) = +TPK%K053(:)*TPK%O3P(:)+TPK%K059(:)*PCONC(:,14)+TPK%K074(:)*PCON&
&C(:,5)+TPK%K077(:)*PCONC(:,1)
!
IF (LHOOK) CALL DR_HOOK('CH_PRODLOSS:SUB1',1,ZHOOK_HANDLE)
RETURN
END SUBROUTINE SUB1
!
!OPTION! -Nfusion
SUBROUTINE SUB2
!
!Indices 21 a 30
!
!
!PPROD(ARO) = +0.10670*K080*<PHO>*<NO2>+1.06698*K081*<PHO>*<HO2>+K082*<ADD>*<NO
!2>+0.02*K083*<ADD>*<O2>+K084*<ADD>*<O3>
 REAL(KIND=JPRB) :: ZHOOK_HANDLE
 IF (LHOOK) CALL DR_HOOK('CH_PRODLOSS:SUB2',0,ZHOOK_HANDLE)
 PPROD(:,21) = +0.10670*TPK%K080(:)*PCONC(:,35)*PCONC(:,4)+1.06698*TPK%K081(:)*&
&PCONC(:,35)*PCONC(:,15)+TPK%K082(:)*PCONC(:,36)*PCONC(:,4)+0.02*TPK%K083(:)*PC&
&ONC(:,36)*TPK%O2(:)+TPK%K084(:)*PCONC(:,36)*PCONC(:,1)
!PLOSS(ARO) = +K060*<OH>+K072*<NO3>
 PLOSS(:,21) = +TPK%K060(:)*PCONC(:,14)+TPK%K072(:)*PCONC(:,5)
!
!PPROD(HCHO) = +K013*<OP1>+0.06517*K016*<CARBO>+0.05*K053*<BIO>*<O3P>+0.00140*K
!057*<ALKA>*<OH>+0.00000*K064*<CARBO>*<OH>+0.35*K065*<OP1>*<OH>+0.02915*K066*<O
!P2>*<OH>+0.57839*K067*<PAN>*<OH>+0.40*K075*<PAN>*<NO3>+0.48290*K076*<ALKE>*<O3
!>+0.90000*K077*<BIO>*<O3>+0.00000*K078*<CARBO>*<O3>+0.70*K079*<PAN>*<O3>+K087*
!<MO2>*<NO>+0.03002*K088*<ALKAP>*<NO>+1.39870*K089*<ALKEP>*<NO>+0.60600*K090*<B
!IOP>*<NO>+0.05848*K092*<CARBOP>*<NO>+0.23419*K093*<OLN>*<NO>+1.33*K101*<MO2>*<
!MO2>+0.80556*K102*<ALKAP>*<MO2>+1.42894*K103*<ALKEP>*<MO2>+1.09000*K104*<BIOP>
!*<MO2>+K105*<AROP>*<MO2>+0.95723*K106*<CARBOP>*<MO2>+0.88625*K107*<OLN>*<MO2>+
!0.07600*K108*<ALKAP>*<CARBOP>+0.68192*K109*<ALKEP>*<CARBOP>+0.34000*K110*<BIOP
!>*<CARBOP>+0.03432*K112*<CARBOP>*<CARBOP>+0.13414*K113*<OLN>*<CARBOP>+0.00000*
!K115*<OLN>*<OLN>+K116*<MO2>*<NO3>+0.03142*K117*<ALKAP>*<NO3>+1.40909*K118*<ALK
!EP>*<NO3>+0.68600*K119*<BIOP>*<NO3>+0.03175*K121*<CARBOP>*<NO3>+0.20740*K122*<
!OLN>*<NO3>+K124*<XO2>*<MO2>
 PPROD(:,22) = +TPK%K013(:)*PCONC(:,28)+0.06517*TPK%K016(:)*PCONC(:,25)+0.05*TP&
&K%K053(:)*PCONC(:,20)*TPK%O3P(:)+0.00140*TPK%K057(:)*PCONC(:,18)*PCONC(:,14)+0&
&.00000*TPK%K064(:)*PCONC(:,25)*PCONC(:,14)+0.35*TPK%K065(:)*PCONC(:,28)*PCONC(&
&:,14)+0.02915*TPK%K066(:)*PCONC(:,29)*PCONC(:,14)+0.57839*TPK%K067(:)*PCONC(:,&
&27)*PCONC(:,14)+0.40*TPK%K075(:)*PCONC(:,27)*PCONC(:,5)+0.48290*TPK%K076(:)*PC&
&ONC(:,19)*PCONC(:,1)+0.90000*TPK%K077(:)*PCONC(:,20)*PCONC(:,1)+0.00000*TPK%K0&
&78(:)*PCONC(:,25)*PCONC(:,1)+0.70*TPK%K079(:)*PCONC(:,27)*PCONC(:,1)+TPK%K087(&
&:)*PCONC(:,31)*PCONC(:,3)+0.03002*TPK%K088(:)*PCONC(:,32)*PCONC(:,3)+1.39870*T&
&PK%K089(:)*PCONC(:,33)*PCONC(:,3)+0.60600*TPK%K090(:)*PCONC(:,34)*PCONC(:,3)+0&
&.05848*TPK%K092(:)*PCONC(:,38)*PCONC(:,3)+0.23419*TPK%K093(:)*PCONC(:,39)*PCON&
&C(:,3)+1.33*TPK%K101(:)*PCONC(:,31)*PCONC(:,31)+0.80556*TPK%K102(:)*PCONC(:,32&
&)*PCONC(:,31)+1.42894*TPK%K103(:)*PCONC(:,33)*PCONC(:,31)+1.09000*TPK%K104(:)*&
&PCONC(:,34)*PCONC(:,31)+TPK%K105(:)*PCONC(:,37)*PCONC(:,31)+0.95723*TPK%K106(:&
&)*PCONC(:,38)*PCONC(:,31)+0.88625*TPK%K107(:)*PCONC(:,39)*PCONC(:,31)+0.07600*&
&TPK%K108(:)*PCONC(:,32)*PCONC(:,38)+0.68192*TPK%K109(:)*PCONC(:,33)*PCONC(:,38&
&)+0.34000*TPK%K110(:)*PCONC(:,34)*PCONC(:,38)+0.03432*TPK%K112(:)*PCONC(:,38)*&
&PCONC(:,38)+0.13414*TPK%K113(:)*PCONC(:,39)*PCONC(:,38)+0.00000*TPK%K115(:)*PC&
&ONC(:,39)*PCONC(:,39)+TPK%K116(:)*PCONC(:,31)*PCONC(:,5)+0.03142*TPK%K117(:)*P&
&CONC(:,32)*PCONC(:,5)+1.40909*TPK%K118(:)*PCONC(:,33)*PCONC(:,5)+0.68600*TPK%K&
&119(:)*PCONC(:,34)*PCONC(:,5)+0.03175*TPK%K121(:)*PCONC(:,38)*PCONC(:,5)+0.207&
&40*TPK%K122(:)*PCONC(:,39)*PCONC(:,5)+TPK%K124(:)*PCONC(:,40)*PCONC(:,31)
!PLOSS(HCHO) = +K010+K011+K061*<OH>+K069*<NO3>
 PLOSS(:,22) = +TPK%K010(:)+TPK%K011(:)+TPK%K061(:)*PCONC(:,14)+TPK%K069(:)*PCO&
&NC(:,5)
!
!PPROD(ALD) = +0.96205*K014*<OP2>+0.20*K017*<ONIT>+K054*<CARBO>*<O3P>+0.08173*K
!057*<ALKA>*<OH>+0.06253*K064*<CARBO>*<OH>+0.07335*K066*<OP2>*<OH>+0.05265*K071
!*<CARBO>*<NO3>+0.51468*K076*<ALKE>*<O3>+0.00000*K077*<BIO>*<O3>+0.15692*K078*<
!CARBO>*<O3>+0.33144*K088*<ALKAP>*<NO>+0.42125*K089*<ALKEP>*<NO>+0.00000*K090*<
!BIOP>*<NO>+0.07368*K092*<CARBOP>*<NO>+1.01182*K093*<OLN>*<NO>+0.56070*K102*<AL
!KAP>*<MO2>+0.46413*K103*<ALKEP>*<MO2>+0.00000*K104*<BIOP>*<MO2>+0.08295*K106*<
!CARBOP>*<MO2>+0.41524*K107*<OLN>*<MO2>+0.71461*K108*<ALKAP>*<CARBOP>+0.68374*K
!109*<ALKEP>*<CARBOP>+0.00000*K110*<BIOP>*<CARBOP>+0.06969*K112*<CARBOP>*<CARBO
!P>+0.42122*K113*<OLN>*<CARBOP>+0.00000*K115*<OLN>*<OLN>+0.33743*K117*<ALKAP>*<
!NO3>+0.43039*K118*<ALKEP>*<NO3>+0.00000*K119*<BIOP>*<NO3>+0.02936*K121*<CARBOP
!>*<NO3>+0.91850*K122*<OLN>*<NO3>
 PPROD(:,23) = +0.96205*TPK%K014(:)*PCONC(:,29)+0.20*TPK%K017(:)*PCONC(:,26)+TP&
&K%K054(:)*PCONC(:,25)*TPK%O3P(:)+0.08173*TPK%K057(:)*PCONC(:,18)*PCONC(:,14)+0&
&.06253*TPK%K064(:)*PCONC(:,25)*PCONC(:,14)+0.07335*TPK%K066(:)*PCONC(:,29)*PCO&
&NC(:,14)+0.05265*TPK%K071(:)*PCONC(:,25)*PCONC(:,5)+0.51468*TPK%K076(:)*PCONC(&
&:,19)*PCONC(:,1)+0.00000*TPK%K077(:)*PCONC(:,20)*PCONC(:,1)+0.15692*TPK%K078(:&
&)*PCONC(:,25)*PCONC(:,1)+0.33144*TPK%K088(:)*PCONC(:,32)*PCONC(:,3)+0.42125*TP&
&K%K089(:)*PCONC(:,33)*PCONC(:,3)+0.00000*TPK%K090(:)*PCONC(:,34)*PCONC(:,3)+0.&
&07368*TPK%K092(:)*PCONC(:,38)*PCONC(:,3)+1.01182*TPK%K093(:)*PCONC(:,39)*PCONC&
&(:,3)+0.56070*TPK%K102(:)*PCONC(:,32)*PCONC(:,31)+0.46413*TPK%K103(:)*PCONC(:,&
&33)*PCONC(:,31)+0.00000*TPK%K104(:)*PCONC(:,34)*PCONC(:,31)+0.08295*TPK%K106(:&
&)*PCONC(:,38)*PCONC(:,31)+0.41524*TPK%K107(:)*PCONC(:,39)*PCONC(:,31)+0.71461*&
&TPK%K108(:)*PCONC(:,32)*PCONC(:,38)+0.68374*TPK%K109(:)*PCONC(:,33)*PCONC(:,38&
&)+0.00000*TPK%K110(:)*PCONC(:,34)*PCONC(:,38)+0.06969*TPK%K112(:)*PCONC(:,38)*&
&PCONC(:,38)+0.42122*TPK%K113(:)*PCONC(:,39)*PCONC(:,38)+0.00000*TPK%K115(:)*PC&
&ONC(:,39)*PCONC(:,39)+0.33743*TPK%K117(:)*PCONC(:,32)*PCONC(:,5)+0.43039*TPK%K&
&118(:)*PCONC(:,33)*PCONC(:,5)+0.00000*TPK%K119(:)*PCONC(:,34)*PCONC(:,5)+0.029&
&36*TPK%K121(:)*PCONC(:,38)*PCONC(:,5)+0.91850*TPK%K122(:)*PCONC(:,39)*PCONC(:,&
&5)
!PLOSS(ALD) = +K012+K062*<OH>+K070*<NO3>
 PLOSS(:,23) = +TPK%K012(:)+TPK%K062(:)*PCONC(:,14)+TPK%K070(:)*PCONC(:,5)
!
!PPROD(KET) = +0.80*K017*<ONIT>+0.03498*K057*<ALKA>*<OH>+0.00853*K064*<CARBO>*<
!OH>+0.37591*K066*<OP2>*<OH>+0.00632*K071*<CARBO>*<NO3>+0.07377*K076*<ALKE>*<O3
!>+0.00000*K077*<BIO>*<O3>+0.54531*K088*<ALKAP>*<NO>+0.05220*K089*<ALKEP>*<NO>+
!0.00000*K090*<BIOP>*<NO>+0.37862*K093*<OLN>*<NO>+0.09673*K102*<ALKAP>*<MO2>+0.
!03814*K103*<ALKEP>*<MO2>+0.00000*K104*<BIOP>*<MO2>+0.09667*K107*<OLN>*<MO2>+0.
!18819*K108*<ALKAP>*<CARBOP>+0.06579*K109*<ALKEP>*<CARBOP>+0.00000*K110*<BIOP>*
!<CARBOP>+0.02190*K112*<CARBOP>*<CARBOP>+0.10822*K113*<OLN>*<CARBOP>+0.00000*K1
!15*<OLN>*<OLN>+0.62978*K117*<ALKAP>*<NO3>+0.02051*K118*<ALKEP>*<NO3>+0.00000*K
!119*<BIOP>*<NO3>+0.34740*K122*<OLN>*<NO3>
 PPROD(:,24) = +0.80*TPK%K017(:)*PCONC(:,26)+0.03498*TPK%K057(:)*PCONC(:,18)*PC&
&ONC(:,14)+0.00853*TPK%K064(:)*PCONC(:,25)*PCONC(:,14)+0.37591*TPK%K066(:)*PCON&
&C(:,29)*PCONC(:,14)+0.00632*TPK%K071(:)*PCONC(:,25)*PCONC(:,5)+0.07377*TPK%K07&
&6(:)*PCONC(:,19)*PCONC(:,1)+0.00000*TPK%K077(:)*PCONC(:,20)*PCONC(:,1)+0.54531&
&*TPK%K088(:)*PCONC(:,32)*PCONC(:,3)+0.05220*TPK%K089(:)*PCONC(:,33)*PCONC(:,3)&
&+0.00000*TPK%K090(:)*PCONC(:,34)*PCONC(:,3)+0.37862*TPK%K093(:)*PCONC(:,39)*PC&
&ONC(:,3)+0.09673*TPK%K102(:)*PCONC(:,32)*PCONC(:,31)+0.03814*TPK%K103(:)*PCONC&
&(:,33)*PCONC(:,31)+0.00000*TPK%K104(:)*PCONC(:,34)*PCONC(:,31)+0.09667*TPK%K10&
&7(:)*PCONC(:,39)*PCONC(:,31)+0.18819*TPK%K108(:)*PCONC(:,32)*PCONC(:,38)+0.065&
&79*TPK%K109(:)*PCONC(:,33)*PCONC(:,38)+0.00000*TPK%K110(:)*PCONC(:,34)*PCONC(:&
&,38)+0.02190*TPK%K112(:)*PCONC(:,38)*PCONC(:,38)+0.10822*TPK%K113(:)*PCONC(:,3&
&9)*PCONC(:,38)+0.00000*TPK%K115(:)*PCONC(:,39)*PCONC(:,39)+0.62978*TPK%K117(:)&
&*PCONC(:,32)*PCONC(:,5)+0.02051*TPK%K118(:)*PCONC(:,33)*PCONC(:,5)+0.00000*TPK&
&%K119(:)*PCONC(:,34)*PCONC(:,5)+0.34740*TPK%K122(:)*PCONC(:,39)*PCONC(:,5)
!PLOSS(KET) = +K015+K063*<OH>
 PLOSS(:,24) = +TPK%K015(:)+TPK%K063(:)*PCONC(:,14)
!
!PPROD(CARBO) = +0.13255*K053*<BIO>*<O3P>+0.00835*K057*<ALKA>*<OH>+0.16919*K064
!*<CARBO>*<OH>+0.21863*K067*<PAN>*<OH>+0.10530*K071*<CARBO>*<NO3>+0.00000*K073*
!<ALKE>*<NO3>+0.91741*K074*<BIO>*<NO3>+0.00000*K076*<ALKE>*<O3>+0.39754*K077*<B
!IO>*<O3>+1.07583*K078*<CARBO>*<O3>+0.03407*K088*<ALKAP>*<NO>+0.45463*K090*<BIO
!P>*<NO>+2.06993*K091*<AROP>*<NO>+0.08670*K092*<CARBOP>*<NO>+0.07976*K102*<ALKA
!P>*<MO2>+0.56064*K104*<BIOP>*<MO2>+1.99461*K105*<AROP>*<MO2>+0.15387*K106*<CAR
!BOP>*<MO2>+0.06954*K108*<ALKAP>*<CARBOP>+0.78591*K110*<BIOP>*<CARBOP>+1.99455*
!K111*<AROP>*<CARBOP>+0.10777*K112*<CARBOP>*<CARBOP>+0.03531*K117*<ALKAP>*<NO3>
!+0.61160*K119*<BIOP>*<NO3>+2.81904*K120*<AROP>*<NO3>+0.03455*K121*<CARBOP>*<NO
!3>
 PPROD(:,25) = +0.13255*TPK%K053(:)*PCONC(:,20)*TPK%O3P(:)+0.00835*TPK%K057(:)*&
&PCONC(:,18)*PCONC(:,14)+0.16919*TPK%K064(:)*PCONC(:,25)*PCONC(:,14)+0.21863*TP&
&K%K067(:)*PCONC(:,27)*PCONC(:,14)+0.10530*TPK%K071(:)*PCONC(:,25)*PCONC(:,5)+0&
&.00000*TPK%K073(:)*PCONC(:,19)*PCONC(:,5)+0.91741*TPK%K074(:)*PCONC(:,20)*PCON&
&C(:,5)+0.00000*TPK%K076(:)*PCONC(:,19)*PCONC(:,1)+0.39754*TPK%K077(:)*PCONC(:,&
&20)*PCONC(:,1)+1.07583*TPK%K078(:)*PCONC(:,25)*PCONC(:,1)+0.03407*TPK%K088(:)*&
&PCONC(:,32)*PCONC(:,3)+0.45463*TPK%K090(:)*PCONC(:,34)*PCONC(:,3)+2.06993*TPK%&
&K091(:)*PCONC(:,37)*PCONC(:,3)+0.08670*TPK%K092(:)*PCONC(:,38)*PCONC(:,3)+0.07&
&976*TPK%K102(:)*PCONC(:,32)*PCONC(:,31)+0.56064*TPK%K104(:)*PCONC(:,34)*PCONC(&
&:,31)+1.99461*TPK%K105(:)*PCONC(:,37)*PCONC(:,31)+0.15387*TPK%K106(:)*PCONC(:,&
&38)*PCONC(:,31)+0.06954*TPK%K108(:)*PCONC(:,32)*PCONC(:,38)+0.78591*TPK%K110(:&
&)*PCONC(:,34)*PCONC(:,38)+1.99455*TPK%K111(:)*PCONC(:,37)*PCONC(:,38)+0.10777*&
&TPK%K112(:)*PCONC(:,38)*PCONC(:,38)+0.03531*TPK%K117(:)*PCONC(:,32)*PCONC(:,5)&
&+0.61160*TPK%K119(:)*PCONC(:,34)*PCONC(:,5)+2.81904*TPK%K120(:)*PCONC(:,37)*PC&
&ONC(:,5)+0.03455*TPK%K121(:)*PCONC(:,38)*PCONC(:,5)
!PLOSS(CARBO) = +K016+K054*<O3P>+K064*<OH>+K071*<NO3>+K078*<O3>
 PLOSS(:,25) = +TPK%K016(:)+TPK%K054(:)*TPK%O3P(:)+TPK%K064(:)*PCONC(:,14)+TPK%&
&K071(:)*PCONC(:,5)+TPK%K078(:)*PCONC(:,1)
!
!PPROD(ONIT) = +0.60*K075*<PAN>*<NO3>+K080*<PHO>*<NO2>+0.08459*K088*<ALKAP>*<NO
!>+0.15300*K090*<BIOP>*<NO>+0.04885*K091*<AROP>*<NO>+0.18401*K093*<OLN>*<NO>+K1
!00*<OLN>*<HO2>+0.67560*K107*<OLN>*<MO2>+0.66562*K113*<OLN>*<CARBOP>+2.00*K114*
!<OLN>*<OLN>+0.00000*K115*<OLN>*<OLN>+0.25928*K122*<OLN>*<NO3>
 PPROD(:,26) = +0.60*TPK%K075(:)*PCONC(:,27)*PCONC(:,5)+TPK%K080(:)*PCONC(:,35)&
&*PCONC(:,4)+0.08459*TPK%K088(:)*PCONC(:,32)*PCONC(:,3)+0.15300*TPK%K090(:)*PCO&
&NC(:,34)*PCONC(:,3)+0.04885*TPK%K091(:)*PCONC(:,37)*PCONC(:,3)+0.18401*TPK%K09&
&3(:)*PCONC(:,39)*PCONC(:,3)+TPK%K100(:)*PCONC(:,39)*PCONC(:,15)+0.67560*TPK%K1&
&07(:)*PCONC(:,39)*PCONC(:,31)+0.66562*TPK%K113(:)*PCONC(:,39)*PCONC(:,38)+2.00&
&*TPK%K114(:)*PCONC(:,39)*PCONC(:,39)+0.00000*TPK%K115(:)*PCONC(:,39)*PCONC(:,3&
&9)+0.25928*TPK%K122(:)*PCONC(:,39)*PCONC(:,5)
!PLOSS(ONIT) = +K017+K068*<OH>
 PLOSS(:,26) = +TPK%K017(:)+TPK%K068(:)*PCONC(:,14)
!
!PPROD(PAN) = +0.28107*K067*<PAN>*<OH>+0.40000*K075*<PAN>*<NO3>+0.30000*K079*<P
!AN>*<O3>+1.00000*K085*<CARBOP>*<NO2>
 PPROD(:,27) = +0.28107*TPK%K067(:)*PCONC(:,27)*PCONC(:,14)+0.40000*TPK%K075(:)&
&*PCONC(:,27)*PCONC(:,5)+0.30000*TPK%K079(:)*PCONC(:,27)*PCONC(:,1)+1.00000*TPK&
&%K085(:)*PCONC(:,38)*PCONC(:,4)
!PLOSS(PAN) = +K067*<OH>+K075*<NO3>+K079*<O3>+K086
 PLOSS(:,27) = +TPK%K067(:)*PCONC(:,14)+TPK%K075(:)*PCONC(:,5)+TPK%K079(:)*PCON&
&C(:,1)+TPK%K086(:)
!
!PPROD(OP1) = +K094*<MO2>*<HO2>
 PPROD(:,28) = +TPK%K094(:)*PCONC(:,31)*PCONC(:,15)
!PLOSS(OP1) = +K013+K065*<OH>
 PLOSS(:,28) = +TPK%K013(:)+TPK%K065(:)*PCONC(:,14)
!
!PPROD(OP2) = +0.10149*K078*<CARBO>*<O3>+1.00524*K095*<ALKAP>*<HO2>+1.00524*K09
!6*<ALKEP>*<HO2>+1.00524*K097*<BIOP>*<HO2>+1.00524*K098*<AROP>*<HO2>+0.80904*K0
!99*<CARBOP>*<HO2>+1.00524*K123*<XO2>*<HO2>
 PPROD(:,29) = +0.10149*TPK%K078(:)*PCONC(:,25)*PCONC(:,1)+1.00524*TPK%K095(:)*&
&PCONC(:,32)*PCONC(:,15)+1.00524*TPK%K096(:)*PCONC(:,33)*PCONC(:,15)+1.00524*TP&
&K%K097(:)*PCONC(:,34)*PCONC(:,15)+1.00524*TPK%K098(:)*PCONC(:,37)*PCONC(:,15)+&
&0.80904*TPK%K099(:)*PCONC(:,38)*PCONC(:,15)+1.00524*TPK%K123(:)*PCONC(:,40)*PC&
&ONC(:,15)
!PLOSS(OP2) = +K014+K066*<OH>
 PLOSS(:,29) = +TPK%K014(:)+TPK%K066(:)*PCONC(:,14)
!
!PPROD(ORA2) = +0.08143*K076*<ALKE>*<O3>+0.00000*K077*<BIO>*<O3>+0.20595*K078*<
!CARBO>*<O3>+0.17307*K099*<CARBOP>*<HO2>+0.13684*K106*<CARBOP>*<MO2>+0.49810*K1
!08*<ALKAP>*<CARBOP>+0.49922*K109*<ALKEP>*<CARBOP>+0.49400*K110*<BIOP>*<CARBOP>
!+0.09955*K112*<CARBOP>*<CARBOP>+0.48963*K113*<OLN>*<CARBOP>
 PPROD(:,30) = +0.08143*TPK%K076(:)*PCONC(:,19)*PCONC(:,1)+0.00000*TPK%K077(:)*&
&PCONC(:,20)*PCONC(:,1)+0.20595*TPK%K078(:)*PCONC(:,25)*PCONC(:,1)+0.17307*TPK%&
&K099(:)*PCONC(:,38)*PCONC(:,15)+0.13684*TPK%K106(:)*PCONC(:,38)*PCONC(:,31)+0.&
&49810*TPK%K108(:)*PCONC(:,32)*PCONC(:,38)+0.49922*TPK%K109(:)*PCONC(:,33)*PCON&
&C(:,38)+0.49400*TPK%K110(:)*PCONC(:,34)*PCONC(:,38)+0.09955*TPK%K112(:)*PCONC(&
&:,38)*PCONC(:,38)+0.48963*TPK%K113(:)*PCONC(:,39)*PCONC(:,38)
!PLOSS(ORA2) = 0.0
 PLOSS(:,30) = 0.0
!
IF (LHOOK) CALL DR_HOOK('CH_PRODLOSS:SUB2',1,ZHOOK_HANDLE)
RETURN
END SUBROUTINE SUB2
!
!OPTION! -Nfusion
SUBROUTINE SUB3
!
!Indices 31 a 40
!
!
!PPROD(MO2) = +K012*<ALD>+0.03795*K014*<OP2>+K055*<CH4>*<OH>+0.65*K065*<OP1>*<O
!H>+0.13966*K076*<ALKE>*<O3>+0.03000*K077*<BIO>*<O3>+0.09016*K088*<ALKAP>*<NO>+
!0.78134*K092*<CARBOP>*<NO>+0.01390*K102*<ALKAP>*<MO2>+0.56031*K106*<CARBOP>*<M
!O2>+0.51480*K108*<ALKAP>*<CARBOP>+0.50078*K109*<ALKEP>*<CARBOP>+0.50600*K110*<
!BIOP>*<CARBOP>+K111*<AROP>*<CARBOP>+1.66702*K112*<CARBOP>*<CARBOP>+0.51037*K11
!3*<OLN>*<CARBOP>+0.09731*K117*<ALKAP>*<NO3>+0.91910*K121*<CARBOP>*<NO3>+K125*<
!XO2>*<CARBOP>
 REAL(KIND=JPRB) :: ZHOOK_HANDLE
 IF (LHOOK) CALL DR_HOOK('CH_PRODLOSS:SUB3',0,ZHOOK_HANDLE)
 PPROD(:,31) = +TPK%K012(:)*PCONC(:,23)+0.03795*TPK%K014(:)*PCONC(:,29)+TPK%K05&
&5(:)*PCONC(:,16)*PCONC(:,14)+0.65*TPK%K065(:)*PCONC(:,28)*PCONC(:,14)+0.13966*&
&TPK%K076(:)*PCONC(:,19)*PCONC(:,1)+0.03000*TPK%K077(:)*PCONC(:,20)*PCONC(:,1)+&
&0.09016*TPK%K088(:)*PCONC(:,32)*PCONC(:,3)+0.78134*TPK%K092(:)*PCONC(:,38)*PCO&
&NC(:,3)+0.01390*TPK%K102(:)*PCONC(:,32)*PCONC(:,31)+0.56031*TPK%K106(:)*PCONC(&
&:,38)*PCONC(:,31)+0.51480*TPK%K108(:)*PCONC(:,32)*PCONC(:,38)+0.50078*TPK%K109&
&(:)*PCONC(:,33)*PCONC(:,38)+0.50600*TPK%K110(:)*PCONC(:,34)*PCONC(:,38)+TPK%K1&
&11(:)*PCONC(:,37)*PCONC(:,38)+1.66702*TPK%K112(:)*PCONC(:,38)*PCONC(:,38)+0.51&
&037*TPK%K113(:)*PCONC(:,39)*PCONC(:,38)+0.09731*TPK%K117(:)*PCONC(:,32)*PCONC(&
&:,5)+0.91910*TPK%K121(:)*PCONC(:,38)*PCONC(:,5)+TPK%K125(:)*PCONC(:,40)*PCONC(&
&:,38)
!PLOSS(MO2) = +K087*<NO>+K094*<HO2>+K101*<MO2>+K101*<MO2>+K102*<ALKAP>+K103*<AL
!KEP>+K104*<BIOP>+K105*<AROP>+K106*<CARBOP>+K107*<OLN>+K116*<NO3>+K124*<XO2>
 PLOSS(:,31) = +TPK%K087(:)*PCONC(:,3)+TPK%K094(:)*PCONC(:,15)+TPK%K101(:)*PCON&
&C(:,31)+TPK%K101(:)*PCONC(:,31)+TPK%K102(:)*PCONC(:,32)+TPK%K103(:)*PCONC(:,33&
&)+TPK%K104(:)*PCONC(:,34)+TPK%K105(:)*PCONC(:,37)+TPK%K106(:)*PCONC(:,38)+TPK%&
&K107(:)*PCONC(:,39)+TPK%K116(:)*PCONC(:,5)+TPK%K124(:)*PCONC(:,40)
!
!PPROD(ALKAP) = +1.00000*K015*<KET>+K056*<ETH>*<OH>+0.87811*K057*<ALKA>*<OH>+0.
!40341*K066*<OP2>*<OH>+1.00000*K068*<ONIT>*<OH>+0.09815*K076*<ALKE>*<O3>+0.0000
!0*K077*<BIO>*<O3>+0.08187*K088*<ALKAP>*<NO>+0.00385*K102*<ALKAP>*<MO2>+0.00828
!*K108*<ALKAP>*<CARBOP>+0.08994*K117*<ALKAP>*<NO3>
 PPROD(:,32) = +1.00000*TPK%K015(:)*PCONC(:,24)+TPK%K056(:)*PCONC(:,17)*PCONC(:&
&,14)+0.87811*TPK%K057(:)*PCONC(:,18)*PCONC(:,14)+0.40341*TPK%K066(:)*PCONC(:,2&
&9)*PCONC(:,14)+1.00000*TPK%K068(:)*PCONC(:,26)*PCONC(:,14)+0.09815*TPK%K076(:)&
&*PCONC(:,19)*PCONC(:,1)+0.00000*TPK%K077(:)*PCONC(:,20)*PCONC(:,1)+0.08187*TPK&
&%K088(:)*PCONC(:,32)*PCONC(:,3)+0.00385*TPK%K102(:)*PCONC(:,32)*PCONC(:,31)+0.&
&00828*TPK%K108(:)*PCONC(:,32)*PCONC(:,38)+0.08994*TPK%K117(:)*PCONC(:,32)*PCON&
&C(:,5)
!PLOSS(ALKAP) = +K088*<NO>+K095*<HO2>+K102*<MO2>+K108*<CARBOP>+K117*<NO3>
 PLOSS(:,32) = +TPK%K088(:)*PCONC(:,3)+TPK%K095(:)*PCONC(:,15)+TPK%K102(:)*PCON&
&C(:,31)+TPK%K108(:)*PCONC(:,38)+TPK%K117(:)*PCONC(:,5)
!
!PPROD(ALKEP) = +1.02529*K058*<ALKE>*<OH>
 PPROD(:,33) = +1.02529*TPK%K058(:)*PCONC(:,19)*PCONC(:,14)
!PLOSS(ALKEP) = +K089*<NO>+K096*<HO2>+K103*<MO2>+K109*<CARBOP>+K118*<NO3>
 PLOSS(:,33) = +TPK%K089(:)*PCONC(:,3)+TPK%K096(:)*PCONC(:,15)+TPK%K103(:)*PCON&
&C(:,31)+TPK%K109(:)*PCONC(:,38)+TPK%K118(:)*PCONC(:,5)
!
!PPROD(BIOP) = +0.00000*K058*<ALKE>*<OH>+1.00000*K059*<BIO>*<OH>
 PPROD(:,34) = +0.00000*TPK%K058(:)*PCONC(:,19)*PCONC(:,14)+1.00000*TPK%K059(:)&
&*PCONC(:,20)*PCONC(:,14)
!PLOSS(BIOP) = +K090*<NO>+K097*<HO2>+K104*<MO2>+K110*<CARBOP>+K119*<NO3>
 PLOSS(:,34) = +TPK%K090(:)*PCONC(:,3)+TPK%K097(:)*PCONC(:,15)+TPK%K104(:)*PCON&
&C(:,31)+TPK%K110(:)*PCONC(:,38)+TPK%K119(:)*PCONC(:,5)
!
!PPROD(PHO) = +0.00276*K060*<ARO>*<OH>+K072*<ARO>*<NO3>
 PPROD(:,35) = +0.00276*TPK%K060(:)*PCONC(:,21)*PCONC(:,14)+TPK%K072(:)*PCONC(:&
&,21)*PCONC(:,5)
!PLOSS(PHO) = +K080*<NO2>+K081*<HO2>
 PLOSS(:,35) = +TPK%K080(:)*PCONC(:,4)+TPK%K081(:)*PCONC(:,15)
!
!PPROD(ADD) = +0.93968*K060*<ARO>*<OH>
 PPROD(:,36) = +0.93968*TPK%K060(:)*PCONC(:,21)*PCONC(:,14)
!PLOSS(ADD) = +K082*<NO2>+K083*<O2>+K084*<O3>
 PLOSS(:,36) = +TPK%K082(:)*PCONC(:,4)+TPK%K083(:)*TPK%O2(:)+TPK%K084(:)*PCONC(&
&:,1)
!
!PPROD(AROP) = +0.98*K083*<ADD>*<O2>
 PPROD(:,37) = +0.98*TPK%K083(:)*PCONC(:,36)*TPK%O2(:)
!PLOSS(AROP) = +K091*<NO>+K098*<HO2>+K105*<MO2>+K111*<CARBOP>+K120*<NO3>
 PLOSS(:,37) = +TPK%K091(:)*PCONC(:,3)+TPK%K098(:)*PCONC(:,15)+TPK%K105(:)*PCON&
&C(:,31)+TPK%K111(:)*PCONC(:,38)+TPK%K120(:)*PCONC(:,5)
!
!PPROD(CARBOP) = +1.00000*K015*<KET>+0.69622*K016*<CARBO>+1.00000*K062*<ALD>*<O
!H>+1.00000*K063*<KET>*<OH>+0.51419*K064*<CARBO>*<OH>+0.05413*K066*<OP2>*<OH>+1
!.00000*K070*<ALD>*<NO3>+0.38881*K071*<CARBO>*<NO3>+0.05705*K076*<ALKE>*<O3>+0.
!17000*K077*<BIO>*<O3>+0.27460*K078*<CARBO>*<O3>+0.70000*K079*<PAN>*<O3>+1.0000
!0*K086*<PAN>+0.09532*K092*<CARBOP>*<NO>+0.05954*K106*<CARBOP>*<MO2>+0.05821*K1
!12*<CARBOP>*<CARBOP>+0.03175*K121*<CARBOP>*<NO3>
 PPROD(:,38) = +1.00000*TPK%K015(:)*PCONC(:,24)+0.69622*TPK%K016(:)*PCONC(:,25)&
&+1.00000*TPK%K062(:)*PCONC(:,23)*PCONC(:,14)+1.00000*TPK%K063(:)*PCONC(:,24)*P&
&CONC(:,14)+0.51419*TPK%K064(:)*PCONC(:,25)*PCONC(:,14)+0.05413*TPK%K066(:)*PCO&
&NC(:,29)*PCONC(:,14)+1.00000*TPK%K070(:)*PCONC(:,23)*PCONC(:,5)+0.38881*TPK%K0&
&71(:)*PCONC(:,25)*PCONC(:,5)+0.05705*TPK%K076(:)*PCONC(:,19)*PCONC(:,1)+0.1700&
&0*TPK%K077(:)*PCONC(:,20)*PCONC(:,1)+0.27460*TPK%K078(:)*PCONC(:,25)*PCONC(:,1&
&)+0.70000*TPK%K079(:)*PCONC(:,27)*PCONC(:,1)+1.00000*TPK%K086(:)*PCONC(:,27)+0&
&.09532*TPK%K092(:)*PCONC(:,38)*PCONC(:,3)+0.05954*TPK%K106(:)*PCONC(:,38)*PCON&
&C(:,31)+0.05821*TPK%K112(:)*PCONC(:,38)*PCONC(:,38)+0.03175*TPK%K121(:)*PCONC(&
&:,38)*PCONC(:,5)
!PLOSS(CARBOP) = +K085*<NO2>+K092*<NO>+K099*<HO2>+K106*<MO2>+K108*<ALKAP>+K109*
!<ALKEP>+K110*<BIOP>+K111*<AROP>+K112*<CARBOP>+K112*<CARBOP>+K113*<OLN>+K121*<N
!O3>+K125*<XO2>
 PLOSS(:,38) = +TPK%K085(:)*PCONC(:,4)+TPK%K092(:)*PCONC(:,3)+TPK%K099(:)*PCONC&
&(:,15)+TPK%K106(:)*PCONC(:,31)+TPK%K108(:)*PCONC(:,32)+TPK%K109(:)*PCONC(:,33)&
&+TPK%K110(:)*PCONC(:,34)+TPK%K111(:)*PCONC(:,37)+TPK%K112(:)*PCONC(:,38)+TPK%K&
&112(:)*PCONC(:,38)+TPK%K113(:)*PCONC(:,39)+TPK%K121(:)*PCONC(:,5)+TPK%K125(:)*&
&PCONC(:,40)
!
!PPROD(OLN) = +0.00000*K071*<CARBO>*<NO3>+0.93768*K073*<ALKE>*<NO3>+1.00000*K07
!4*<BIO>*<NO3>
 PPROD(:,39) = +0.00000*TPK%K071(:)*PCONC(:,25)*PCONC(:,5)+0.93768*TPK%K073(:)*&
&PCONC(:,19)*PCONC(:,5)+1.00000*TPK%K074(:)*PCONC(:,20)*PCONC(:,5)
!PLOSS(OLN) = +K093*<NO>+K100*<HO2>+K107*<MO2>+K113*<CARBOP>+K114*<OLN>+K114*<O
!LN>+K115*<OLN>+K115*<OLN>+K122*<NO3>
 PLOSS(:,39) = +TPK%K093(:)*PCONC(:,3)+TPK%K100(:)*PCONC(:,15)+TPK%K107(:)*PCON&
&C(:,31)+TPK%K113(:)*PCONC(:,38)+TPK%K114(:)*PCONC(:,39)+TPK%K114(:)*PCONC(:,39&
&)+TPK%K115(:)*PCONC(:,39)+TPK%K115(:)*PCONC(:,39)+TPK%K122(:)*PCONC(:,5)
!
!PPROD(XO2) = +0.15*K053*<BIO>*<O3P>+0.10318*K060*<ARO>*<OH>+0.10162*K064*<CARB
!O>*<OH>+0.09333*K066*<OP2>*<OH>+K067*<PAN>*<OH>+0.10530*K071*<CARBO>*<NO3>+K07
!5*<PAN>*<NO3>+0.00000*K076*<ALKE>*<O3>+0.13000*K077*<BIO>*<O3>+0.13007*K088*<A
!LKAP>*<NO>+0.02563*K092*<CARBOP>*<NO>+0.13370*K102*<ALKAP>*<MO2>+0.02212*K106*
!<CARBOP>*<MO2>+0.11306*K108*<ALKAP>*<CARBOP>+0.01593*K112*<CARBOP>*<CARBOP>+0.
!16271*K117*<ALKAP>*<NO3>+0.01021*K121*<CARBOP>*<NO3>
 PPROD(:,40) = +0.15*TPK%K053(:)*PCONC(:,20)*TPK%O3P(:)+0.10318*TPK%K060(:)*PCO&
&NC(:,21)*PCONC(:,14)+0.10162*TPK%K064(:)*PCONC(:,25)*PCONC(:,14)+0.09333*TPK%K&
&066(:)*PCONC(:,29)*PCONC(:,14)+TPK%K067(:)*PCONC(:,27)*PCONC(:,14)+0.10530*TPK&
&%K071(:)*PCONC(:,25)*PCONC(:,5)+TPK%K075(:)*PCONC(:,27)*PCONC(:,5)+0.00000*TPK&
&%K076(:)*PCONC(:,19)*PCONC(:,1)+0.13000*TPK%K077(:)*PCONC(:,20)*PCONC(:,1)+0.1&
&3007*TPK%K088(:)*PCONC(:,32)*PCONC(:,3)+0.02563*TPK%K092(:)*PCONC(:,38)*PCONC(&
&:,3)+0.13370*TPK%K102(:)*PCONC(:,32)*PCONC(:,31)+0.02212*TPK%K106(:)*PCONC(:,3&
&8)*PCONC(:,31)+0.11306*TPK%K108(:)*PCONC(:,32)*PCONC(:,38)+0.01593*TPK%K112(:)&
&*PCONC(:,38)*PCONC(:,38)+0.16271*TPK%K117(:)*PCONC(:,32)*PCONC(:,5)+0.01021*TP&
&K%K121(:)*PCONC(:,38)*PCONC(:,5)
!PLOSS(XO2) = +K123*<HO2>+K124*<MO2>+K125*<CARBOP>+K126*<XO2>+K126*<XO2>+K127*<
!NO>+K128*<NO3>
 PLOSS(:,40) = +TPK%K123(:)*PCONC(:,15)+TPK%K124(:)*PCONC(:,31)+TPK%K125(:)*PCO&
&NC(:,38)+TPK%K126(:)*PCONC(:,40)+TPK%K126(:)*PCONC(:,40)+TPK%K127(:)*PCONC(:,3&
&)+TPK%K128(:)*PCONC(:,5)
!
IF (LHOOK) CALL DR_HOOK('CH_PRODLOSS:SUB3',1,ZHOOK_HANDLE)
RETURN
END SUBROUTINE SUB3
!
!
END SUBROUTINE CH_PRODLOSS
