!     ######spl
      SUBROUTINE CH_SPARSE(KSPARSE, KSPARSEDIM)
      USE PARKIND1, ONLY : JPRB
      USE YOMHOOK , ONLY : LHOOK, DR_HOOK
!!    #########################################
!!
!!*** *MODD_CH_SPARSE*
!!
!!    PURPOSE
!!    -------
!       calculation of the non-zero matrix elements in the Jacobian
!!
!!**  METHOD
!!    ------
!!      A 2D array KSPARSE of DIMESNSION(2,*) is returned, containing 
!!    the indices of the non-zero matrix elements in the Jacobian.
!!
!!    REFERENCE
!!    ---------
!!    none
!!
!!    AUTHOR
!!    ------
!!    Karsten Suhre (LA)
!!
!!    MODIFICATIONS
!!    -------------
!!    Original 26/07/96
!!    Modified 05/05/98: Vectorization (Vincent Crassier & KS)
!!
!!----------------------------------------------------------------------
!!
!!    EXTERNAL
!!    --------
!!    none
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!!    none
!!
!!    EXPLICIT ARGUMENTS
!!    ------------------
IMPLICIT NONE
INTEGER, INTENT(INOUT)                        :: KSPARSEDIM
INTEGER, INTENT(OUT), DIMENSION(2,KSPARSEDIM) :: KSPARSE
!!
!!    LOCAL VARIABLES
!!    ---------------
!!    none
!!----------------------------------------------------------------------
!!
!!    EXECUTABLE STATEMENTS
!!    ---------------------
! check if output array is large enough
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('CH_SPARSE',0,ZHOOK_HANDLE)
IF (KSPARSEDIM.LT.432) THEN
  STOP 'CH_SPARSE ERROR: array KSPARSE is too small!'
END IF
!O3/O3
 KSPARSE(1, 1)=1
 KSPARSE(2, 1)=1
!O3/NO
 KSPARSE(1, 2)=1
 KSPARSE(2, 2)=3
!O3/NO2
 KSPARSE(1, 3)=1
 KSPARSE(2, 3)=4
!O3/OH
 KSPARSE(1, 4)=1
 KSPARSE(2, 4)=14
!O3/HO2
 KSPARSE(1, 5)=1
 KSPARSE(2, 5)=15
!O3/ALKE
 KSPARSE(1, 6)=1
 KSPARSE(2, 6)=19
!O3/BIO
 KSPARSE(1, 7)=1
 KSPARSE(2, 7)=20
!O3/CARBO
 KSPARSE(1, 8)=1
 KSPARSE(2, 8)=25
!O3/PAN
 KSPARSE(1, 9)=1
 KSPARSE(2, 9)=27
!O3/ADD
 KSPARSE(1, 10)=1
 KSPARSE(2, 10)=36
!O3/CARBOP
 KSPARSE(1, 11)=1
 KSPARSE(2, 11)=38
!H2O2/O3
 KSPARSE(1, 12)=2
 KSPARSE(2, 12)=1
!H2O2/H2O2
 KSPARSE(1, 13)=2
 KSPARSE(2, 13)=2
!H2O2/OH
 KSPARSE(1, 14)=2
 KSPARSE(2, 14)=14
!H2O2/HO2
 KSPARSE(1, 15)=2
 KSPARSE(2, 15)=15
!H2O2/ALKE
 KSPARSE(1, 16)=2
 KSPARSE(2, 16)=19
!H2O2/BIO
 KSPARSE(1, 17)=2
 KSPARSE(2, 17)=20
!NO/O3
 KSPARSE(1, 18)=3
 KSPARSE(2, 18)=1
!NO/NO
 KSPARSE(1, 19)=3
 KSPARSE(2, 19)=3
!NO/NO2
 KSPARSE(1, 20)=3
 KSPARSE(2, 20)=4
!NO/NO3
 KSPARSE(1, 21)=3
 KSPARSE(2, 21)=5
!NO/HONO
 KSPARSE(1, 22)=3
 KSPARSE(2, 22)=7
!NO/OH
 KSPARSE(1, 23)=3
 KSPARSE(2, 23)=14
!NO/HO2
 KSPARSE(1, 24)=3
 KSPARSE(2, 24)=15
!NO/MO2
 KSPARSE(1, 25)=3
 KSPARSE(2, 25)=31
!NO/ALKAP
 KSPARSE(1, 26)=3
 KSPARSE(2, 26)=32
!NO/ALKEP
 KSPARSE(1, 27)=3
 KSPARSE(2, 27)=33
!NO/BIOP
 KSPARSE(1, 28)=3
 KSPARSE(2, 28)=34
!NO/AROP
 KSPARSE(1, 29)=3
 KSPARSE(2, 29)=37
!NO/CARBOP
 KSPARSE(1, 30)=3
 KSPARSE(2, 30)=38
!NO/OLN
 KSPARSE(1, 31)=3
 KSPARSE(2, 31)=39
!NO/XO2
 KSPARSE(1, 32)=3
 KSPARSE(2, 32)=40
!NO2/O3
 KSPARSE(1, 33)=4
 KSPARSE(2, 33)=1
!NO2/NO
 KSPARSE(1, 34)=4
 KSPARSE(2, 34)=3
!NO2/NO2
 KSPARSE(1, 35)=4
 KSPARSE(2, 35)=4
!NO2/NO3
 KSPARSE(1, 36)=4
 KSPARSE(2, 36)=5
!NO2/N2O5
 KSPARSE(1, 37)=4
 KSPARSE(2, 37)=6
!NO2/HONO
 KSPARSE(1, 38)=4
 KSPARSE(2, 38)=7
!NO2/HNO3
 KSPARSE(1, 39)=4
 KSPARSE(2, 39)=8
!NO2/HNO4
 KSPARSE(1, 40)=4
 KSPARSE(2, 40)=9
!NO2/OH
 KSPARSE(1, 41)=4
 KSPARSE(2, 41)=14
!NO2/HO2
 KSPARSE(1, 42)=4
 KSPARSE(2, 42)=15
!NO2/CARBO
 KSPARSE(1, 43)=4
 KSPARSE(2, 43)=25
!NO2/ONIT
 KSPARSE(1, 44)=4
 KSPARSE(2, 44)=26
!NO2/PAN
 KSPARSE(1, 45)=4
 KSPARSE(2, 45)=27
!NO2/MO2
 KSPARSE(1, 46)=4
 KSPARSE(2, 46)=31
!NO2/ALKAP
 KSPARSE(1, 47)=4
 KSPARSE(2, 47)=32
!NO2/ALKEP
 KSPARSE(1, 48)=4
 KSPARSE(2, 48)=33
!NO2/BIOP
 KSPARSE(1, 49)=4
 KSPARSE(2, 49)=34
!NO2/PHO
 KSPARSE(1, 50)=4
 KSPARSE(2, 50)=35
!NO2/ADD
 KSPARSE(1, 51)=4
 KSPARSE(2, 51)=36
!NO2/AROP
 KSPARSE(1, 52)=4
 KSPARSE(2, 52)=37
!NO2/CARBOP
 KSPARSE(1, 53)=4
 KSPARSE(2, 53)=38
!NO2/OLN
 KSPARSE(1, 54)=4
 KSPARSE(2, 54)=39
!NO2/XO2
 KSPARSE(1, 55)=4
 KSPARSE(2, 55)=40
!NO3/O3
 KSPARSE(1, 56)=5
 KSPARSE(2, 56)=1
!NO3/NO
 KSPARSE(1, 57)=5
 KSPARSE(2, 57)=3
!NO3/NO2
 KSPARSE(1, 58)=5
 KSPARSE(2, 58)=4
!NO3/NO3
 KSPARSE(1, 59)=5
 KSPARSE(2, 59)=5
!NO3/N2O5
 KSPARSE(1, 60)=5
 KSPARSE(2, 60)=6
!NO3/HNO3
 KSPARSE(1, 61)=5
 KSPARSE(2, 61)=8
!NO3/HNO4
 KSPARSE(1, 62)=5
 KSPARSE(2, 62)=9
!NO3/OH
 KSPARSE(1, 63)=5
 KSPARSE(2, 63)=14
!NO3/HO2
 KSPARSE(1, 64)=5
 KSPARSE(2, 64)=15
!NO3/ALKE
 KSPARSE(1, 65)=5
 KSPARSE(2, 65)=19
!NO3/BIO
 KSPARSE(1, 66)=5
 KSPARSE(2, 66)=20
!NO3/ARO
 KSPARSE(1, 67)=5
 KSPARSE(2, 67)=21
!NO3/HCHO
 KSPARSE(1, 68)=5
 KSPARSE(2, 68)=22
!NO3/ALD
 KSPARSE(1, 69)=5
 KSPARSE(2, 69)=23
!NO3/CARBO
 KSPARSE(1, 70)=5
 KSPARSE(2, 70)=25
!NO3/PAN
 KSPARSE(1, 71)=5
 KSPARSE(2, 71)=27
!NO3/MO2
 KSPARSE(1, 72)=5
 KSPARSE(2, 72)=31
!NO3/ALKAP
 KSPARSE(1, 73)=5
 KSPARSE(2, 73)=32
!NO3/ALKEP
 KSPARSE(1, 74)=5
 KSPARSE(2, 74)=33
!NO3/BIOP
 KSPARSE(1, 75)=5
 KSPARSE(2, 75)=34
!NO3/AROP
 KSPARSE(1, 76)=5
 KSPARSE(2, 76)=37
!NO3/CARBOP
 KSPARSE(1, 77)=5
 KSPARSE(2, 77)=38
!NO3/OLN
 KSPARSE(1, 78)=5
 KSPARSE(2, 78)=39
!NO3/XO2
 KSPARSE(1, 79)=5
 KSPARSE(2, 79)=40
!N2O5/NO2
 KSPARSE(1, 80)=6
 KSPARSE(2, 80)=4
!N2O5/NO3
 KSPARSE(1, 81)=6
 KSPARSE(2, 81)=5
!N2O5/N2O5
 KSPARSE(1, 82)=6
 KSPARSE(2, 82)=6
!HONO/NO
 KSPARSE(1, 83)=7
 KSPARSE(2, 83)=3
!HONO/NO2
 KSPARSE(1, 84)=7
 KSPARSE(2, 84)=4
!HONO/HONO
 KSPARSE(1, 85)=7
 KSPARSE(2, 85)=7
!HONO/OH
 KSPARSE(1, 86)=7
 KSPARSE(2, 86)=14
!HONO/ADD
 KSPARSE(1, 87)=7
 KSPARSE(2, 87)=36
!HNO3/NO2
 KSPARSE(1, 88)=8
 KSPARSE(2, 88)=4
!HNO3/NO3
 KSPARSE(1, 89)=8
 KSPARSE(2, 89)=5
!HNO3/HNO3
 KSPARSE(1, 90)=8
 KSPARSE(2, 90)=8
!HNO3/OH
 KSPARSE(1, 91)=8
 KSPARSE(2, 91)=14
!HNO3/HO2
 KSPARSE(1, 92)=8
 KSPARSE(2, 92)=15
!HNO3/ARO
 KSPARSE(1, 93)=8
 KSPARSE(2, 93)=21
!HNO3/HCHO
 KSPARSE(1, 94)=8
 KSPARSE(2, 94)=22
!HNO3/ALD
 KSPARSE(1, 95)=8
 KSPARSE(2, 95)=23
!HNO3/CARBO
 KSPARSE(1, 96)=8
 KSPARSE(2, 96)=25
!HNO4/NO2
 KSPARSE(1, 97)=9
 KSPARSE(2, 97)=4
!HNO4/HNO4
 KSPARSE(1, 98)=9
 KSPARSE(2, 98)=9
!HNO4/OH
 KSPARSE(1, 99)=9
 KSPARSE(2, 99)=14
!HNO4/HO2
 KSPARSE(1, 100)=9
 KSPARSE(2, 100)=15
!SO2/SO2
 KSPARSE(1, 101)=11
 KSPARSE(2, 101)=11
!SO2/OH
 KSPARSE(1, 102)=11
 KSPARSE(2, 102)=14
!SULF/SO2
 KSPARSE(1, 103)=12
 KSPARSE(2, 103)=11
!SULF/OH
 KSPARSE(1, 104)=12
 KSPARSE(2, 104)=14
!CO/O3
 KSPARSE(1, 105)=13
 KSPARSE(2, 105)=1
!CO/NO3
 KSPARSE(1, 106)=13
 KSPARSE(2, 106)=5
!CO/CO
 KSPARSE(1, 107)=13
 KSPARSE(2, 107)=13
!CO/OH
 KSPARSE(1, 108)=13
 KSPARSE(2, 108)=14
!CO/ALKA
 KSPARSE(1, 109)=13
 KSPARSE(2, 109)=18
!CO/ALKE
 KSPARSE(1, 110)=13
 KSPARSE(2, 110)=19
!CO/BIO
 KSPARSE(1, 111)=13
 KSPARSE(2, 111)=20
!CO/HCHO
 KSPARSE(1, 112)=13
 KSPARSE(2, 112)=22
!CO/ALD
 KSPARSE(1, 113)=13
 KSPARSE(2, 113)=23
!CO/CARBO
 KSPARSE(1, 114)=13
 KSPARSE(2, 114)=25
!CO/PAN
 KSPARSE(1, 115)=13
 KSPARSE(2, 115)=27
!OH/O3
 KSPARSE(1, 116)=14
 KSPARSE(2, 116)=1
!OH/H2O2
 KSPARSE(1, 117)=14
 KSPARSE(2, 117)=2
!OH/NO
 KSPARSE(1, 118)=14
 KSPARSE(2, 118)=3
!OH/NO2
 KSPARSE(1, 119)=14
 KSPARSE(2, 119)=4
!OH/NO3
 KSPARSE(1, 120)=14
 KSPARSE(2, 120)=5
!OH/HONO
 KSPARSE(1, 121)=14
 KSPARSE(2, 121)=7
!OH/HNO3
 KSPARSE(1, 122)=14
 KSPARSE(2, 122)=8
!OH/HNO4
 KSPARSE(1, 123)=14
 KSPARSE(2, 123)=9
!OH/SO2
 KSPARSE(1, 124)=14
 KSPARSE(2, 124)=11
!OH/CO
 KSPARSE(1, 125)=14
 KSPARSE(2, 125)=13
!OH/OH
 KSPARSE(1, 126)=14
 KSPARSE(2, 126)=14
!OH/HO2
 KSPARSE(1, 127)=14
 KSPARSE(2, 127)=15
!OH/CH4
 KSPARSE(1, 128)=14
 KSPARSE(2, 128)=16
!OH/ETH
 KSPARSE(1, 129)=14
 KSPARSE(2, 129)=17
!OH/ALKA
 KSPARSE(1, 130)=14
 KSPARSE(2, 130)=18
!OH/ALKE
 KSPARSE(1, 131)=14
 KSPARSE(2, 131)=19
!OH/BIO
 KSPARSE(1, 132)=14
 KSPARSE(2, 132)=20
!OH/ARO
 KSPARSE(1, 133)=14
 KSPARSE(2, 133)=21
!OH/HCHO
 KSPARSE(1, 134)=14
 KSPARSE(2, 134)=22
!OH/ALD
 KSPARSE(1, 135)=14
 KSPARSE(2, 135)=23
!OH/KET
 KSPARSE(1, 136)=14
 KSPARSE(2, 136)=24
!OH/CARBO
 KSPARSE(1, 137)=14
 KSPARSE(2, 137)=25
!OH/ONIT
 KSPARSE(1, 138)=14
 KSPARSE(2, 138)=26
!OH/PAN
 KSPARSE(1, 139)=14
 KSPARSE(2, 139)=27
!OH/OP1
 KSPARSE(1, 140)=14
 KSPARSE(2, 140)=28
!OH/OP2
 KSPARSE(1, 141)=14
 KSPARSE(2, 141)=29
!OH/ADD
 KSPARSE(1, 142)=14
 KSPARSE(2, 142)=36
!HO2/O3
 KSPARSE(1, 143)=15
 KSPARSE(2, 143)=1
!HO2/H2O2
 KSPARSE(1, 144)=15
 KSPARSE(2, 144)=2
!HO2/NO
 KSPARSE(1, 145)=15
 KSPARSE(2, 145)=3
!HO2/NO2
 KSPARSE(1, 146)=15
 KSPARSE(2, 146)=4
!HO2/NO3
 KSPARSE(1, 147)=15
 KSPARSE(2, 147)=5
!HO2/HNO4
 KSPARSE(1, 148)=15
 KSPARSE(2, 148)=9
!HO2/SO2
 KSPARSE(1, 149)=15
 KSPARSE(2, 149)=11
!HO2/CO
 KSPARSE(1, 150)=15
 KSPARSE(2, 150)=13
!HO2/OH
 KSPARSE(1, 151)=15
 KSPARSE(2, 151)=14
!HO2/HO2
 KSPARSE(1, 152)=15
 KSPARSE(2, 152)=15
!HO2/ALKA
 KSPARSE(1, 153)=15
 KSPARSE(2, 153)=18
!HO2/ALKE
 KSPARSE(1, 154)=15
 KSPARSE(2, 154)=19
!HO2/BIO
 KSPARSE(1, 155)=15
 KSPARSE(2, 155)=20
!HO2/ARO
 KSPARSE(1, 156)=15
 KSPARSE(2, 156)=21
!HO2/HCHO
 KSPARSE(1, 157)=15
 KSPARSE(2, 157)=22
!HO2/ALD
 KSPARSE(1, 158)=15
 KSPARSE(2, 158)=23
!HO2/CARBO
 KSPARSE(1, 159)=15
 KSPARSE(2, 159)=25
!HO2/ONIT
 KSPARSE(1, 160)=15
 KSPARSE(2, 160)=26
!HO2/PAN
 KSPARSE(1, 161)=15
 KSPARSE(2, 161)=27
!HO2/OP1
 KSPARSE(1, 162)=15
 KSPARSE(2, 162)=28
!HO2/OP2
 KSPARSE(1, 163)=15
 KSPARSE(2, 163)=29
!HO2/MO2
 KSPARSE(1, 164)=15
 KSPARSE(2, 164)=31
!HO2/ALKAP
 KSPARSE(1, 165)=15
 KSPARSE(2, 165)=32
!HO2/ALKEP
 KSPARSE(1, 166)=15
 KSPARSE(2, 166)=33
!HO2/BIOP
 KSPARSE(1, 167)=15
 KSPARSE(2, 167)=34
!HO2/PHO
 KSPARSE(1, 168)=15
 KSPARSE(2, 168)=35
!HO2/ADD
 KSPARSE(1, 169)=15
 KSPARSE(2, 169)=36
!HO2/AROP
 KSPARSE(1, 170)=15
 KSPARSE(2, 170)=37
!HO2/CARBOP
 KSPARSE(1, 171)=15
 KSPARSE(2, 171)=38
!HO2/OLN
 KSPARSE(1, 172)=15
 KSPARSE(2, 172)=39
!HO2/XO2
 KSPARSE(1, 173)=15
 KSPARSE(2, 173)=40
!CH4/O3
 KSPARSE(1, 174)=16
 KSPARSE(2, 174)=1
!CH4/OH
 KSPARSE(1, 175)=16
 KSPARSE(2, 175)=14
!CH4/CH4
 KSPARSE(1, 176)=16
 KSPARSE(2, 176)=16
!CH4/ALKE
 KSPARSE(1, 177)=16
 KSPARSE(2, 177)=19
!ETH/O3
 KSPARSE(1, 178)=17
 KSPARSE(2, 178)=1
!ETH/OH
 KSPARSE(1, 179)=17
 KSPARSE(2, 179)=14
!ETH/ETH
 KSPARSE(1, 180)=17
 KSPARSE(2, 180)=17
!ETH/ALKE
 KSPARSE(1, 181)=17
 KSPARSE(2, 181)=19
!ALKA/OH
 KSPARSE(1, 182)=18
 KSPARSE(2, 182)=14
!ALKA/ALKA
 KSPARSE(1, 183)=18
 KSPARSE(2, 183)=18
!ALKE/O3
 KSPARSE(1, 184)=19
 KSPARSE(2, 184)=1
!ALKE/NO
 KSPARSE(1, 185)=19
 KSPARSE(2, 185)=3
!ALKE/NO3
 KSPARSE(1, 186)=19
 KSPARSE(2, 186)=5
!ALKE/OH
 KSPARSE(1, 187)=19
 KSPARSE(2, 187)=14
!ALKE/ALKE
 KSPARSE(1, 188)=19
 KSPARSE(2, 188)=19
!ALKE/BIO
 KSPARSE(1, 189)=19
 KSPARSE(2, 189)=20
!ALKE/MO2
 KSPARSE(1, 190)=19
 KSPARSE(2, 190)=31
!ALKE/BIOP
 KSPARSE(1, 191)=19
 KSPARSE(2, 191)=34
!ALKE/CARBOP
 KSPARSE(1, 192)=19
 KSPARSE(2, 192)=38
!BIO/O3
 KSPARSE(1, 193)=20
 KSPARSE(2, 193)=1
!BIO/NO3
 KSPARSE(1, 194)=20
 KSPARSE(2, 194)=5
!BIO/OH
 KSPARSE(1, 195)=20
 KSPARSE(2, 195)=14
!BIO/BIO
 KSPARSE(1, 196)=20
 KSPARSE(2, 196)=20
!ARO/O3
 KSPARSE(1, 197)=21
 KSPARSE(2, 197)=1
!ARO/NO2
 KSPARSE(1, 198)=21
 KSPARSE(2, 198)=4
!ARO/NO3
 KSPARSE(1, 199)=21
 KSPARSE(2, 199)=5
!ARO/OH
 KSPARSE(1, 200)=21
 KSPARSE(2, 200)=14
!ARO/HO2
 KSPARSE(1, 201)=21
 KSPARSE(2, 201)=15
!ARO/ARO
 KSPARSE(1, 202)=21
 KSPARSE(2, 202)=21
!ARO/PHO
 KSPARSE(1, 203)=21
 KSPARSE(2, 203)=35
!ARO/ADD
 KSPARSE(1, 204)=21
 KSPARSE(2, 204)=36
!HCHO/O3
 KSPARSE(1, 205)=22
 KSPARSE(2, 205)=1
!HCHO/NO
 KSPARSE(1, 206)=22
 KSPARSE(2, 206)=3
!HCHO/NO3
 KSPARSE(1, 207)=22
 KSPARSE(2, 207)=5
!HCHO/OH
 KSPARSE(1, 208)=22
 KSPARSE(2, 208)=14
!HCHO/ALKA
 KSPARSE(1, 209)=22
 KSPARSE(2, 209)=18
!HCHO/ALKE
 KSPARSE(1, 210)=22
 KSPARSE(2, 210)=19
!HCHO/BIO
 KSPARSE(1, 211)=22
 KSPARSE(2, 211)=20
!HCHO/HCHO
 KSPARSE(1, 212)=22
 KSPARSE(2, 212)=22
!HCHO/CARBO
 KSPARSE(1, 213)=22
 KSPARSE(2, 213)=25
!HCHO/PAN
 KSPARSE(1, 214)=22
 KSPARSE(2, 214)=27
!HCHO/OP1
 KSPARSE(1, 215)=22
 KSPARSE(2, 215)=28
!HCHO/OP2
 KSPARSE(1, 216)=22
 KSPARSE(2, 216)=29
!HCHO/MO2
 KSPARSE(1, 217)=22
 KSPARSE(2, 217)=31
!HCHO/ALKAP
 KSPARSE(1, 218)=22
 KSPARSE(2, 218)=32
!HCHO/ALKEP
 KSPARSE(1, 219)=22
 KSPARSE(2, 219)=33
!HCHO/BIOP
 KSPARSE(1, 220)=22
 KSPARSE(2, 220)=34
!HCHO/AROP
 KSPARSE(1, 221)=22
 KSPARSE(2, 221)=37
!HCHO/CARBOP
 KSPARSE(1, 222)=22
 KSPARSE(2, 222)=38
!HCHO/OLN
 KSPARSE(1, 223)=22
 KSPARSE(2, 223)=39
!HCHO/XO2
 KSPARSE(1, 224)=22
 KSPARSE(2, 224)=40
!ALD/O3
 KSPARSE(1, 225)=23
 KSPARSE(2, 225)=1
!ALD/NO
 KSPARSE(1, 226)=23
 KSPARSE(2, 226)=3
!ALD/NO3
 KSPARSE(1, 227)=23
 KSPARSE(2, 227)=5
!ALD/OH
 KSPARSE(1, 228)=23
 KSPARSE(2, 228)=14
!ALD/ALKA
 KSPARSE(1, 229)=23
 KSPARSE(2, 229)=18
!ALD/ALKE
 KSPARSE(1, 230)=23
 KSPARSE(2, 230)=19
!ALD/BIO
 KSPARSE(1, 231)=23
 KSPARSE(2, 231)=20
!ALD/ALD
 KSPARSE(1, 232)=23
 KSPARSE(2, 232)=23
!ALD/CARBO
 KSPARSE(1, 233)=23
 KSPARSE(2, 233)=25
!ALD/ONIT
 KSPARSE(1, 234)=23
 KSPARSE(2, 234)=26
!ALD/OP2
 KSPARSE(1, 235)=23
 KSPARSE(2, 235)=29
!ALD/MO2
 KSPARSE(1, 236)=23
 KSPARSE(2, 236)=31
!ALD/ALKAP
 KSPARSE(1, 237)=23
 KSPARSE(2, 237)=32
!ALD/ALKEP
 KSPARSE(1, 238)=23
 KSPARSE(2, 238)=33
!ALD/BIOP
 KSPARSE(1, 239)=23
 KSPARSE(2, 239)=34
!ALD/CARBOP
 KSPARSE(1, 240)=23
 KSPARSE(2, 240)=38
!ALD/OLN
 KSPARSE(1, 241)=23
 KSPARSE(2, 241)=39
!KET/O3
 KSPARSE(1, 242)=24
 KSPARSE(2, 242)=1
!KET/NO
 KSPARSE(1, 243)=24
 KSPARSE(2, 243)=3
!KET/NO3
 KSPARSE(1, 244)=24
 KSPARSE(2, 244)=5
!KET/OH
 KSPARSE(1, 245)=24
 KSPARSE(2, 245)=14
!KET/ALKA
 KSPARSE(1, 246)=24
 KSPARSE(2, 246)=18
!KET/ALKE
 KSPARSE(1, 247)=24
 KSPARSE(2, 247)=19
!KET/BIO
 KSPARSE(1, 248)=24
 KSPARSE(2, 248)=20
!KET/KET
 KSPARSE(1, 249)=24
 KSPARSE(2, 249)=24
!KET/CARBO
 KSPARSE(1, 250)=24
 KSPARSE(2, 250)=25
!KET/ONIT
 KSPARSE(1, 251)=24
 KSPARSE(2, 251)=26
!KET/OP2
 KSPARSE(1, 252)=24
 KSPARSE(2, 252)=29
!KET/MO2
 KSPARSE(1, 253)=24
 KSPARSE(2, 253)=31
!KET/ALKAP
 KSPARSE(1, 254)=24
 KSPARSE(2, 254)=32
!KET/ALKEP
 KSPARSE(1, 255)=24
 KSPARSE(2, 255)=33
!KET/BIOP
 KSPARSE(1, 256)=24
 KSPARSE(2, 256)=34
!KET/CARBOP
 KSPARSE(1, 257)=24
 KSPARSE(2, 257)=38
!KET/OLN
 KSPARSE(1, 258)=24
 KSPARSE(2, 258)=39
!CARBO/O3
 KSPARSE(1, 259)=25
 KSPARSE(2, 259)=1
!CARBO/NO
 KSPARSE(1, 260)=25
 KSPARSE(2, 260)=3
!CARBO/NO3
 KSPARSE(1, 261)=25
 KSPARSE(2, 261)=5
!CARBO/OH
 KSPARSE(1, 262)=25
 KSPARSE(2, 262)=14
!CARBO/ALKA
 KSPARSE(1, 263)=25
 KSPARSE(2, 263)=18
!CARBO/ALKE
 KSPARSE(1, 264)=25
 KSPARSE(2, 264)=19
!CARBO/BIO
 KSPARSE(1, 265)=25
 KSPARSE(2, 265)=20
!CARBO/CARBO
 KSPARSE(1, 266)=25
 KSPARSE(2, 266)=25
!CARBO/PAN
 KSPARSE(1, 267)=25
 KSPARSE(2, 267)=27
!CARBO/MO2
 KSPARSE(1, 268)=25
 KSPARSE(2, 268)=31
!CARBO/ALKAP
 KSPARSE(1, 269)=25
 KSPARSE(2, 269)=32
!CARBO/BIOP
 KSPARSE(1, 270)=25
 KSPARSE(2, 270)=34
!CARBO/AROP
 KSPARSE(1, 271)=25
 KSPARSE(2, 271)=37
!CARBO/CARBOP
 KSPARSE(1, 272)=25
 KSPARSE(2, 272)=38
!ONIT/NO
 KSPARSE(1, 273)=26
 KSPARSE(2, 273)=3
!ONIT/NO2
 KSPARSE(1, 274)=26
 KSPARSE(2, 274)=4
!ONIT/NO3
 KSPARSE(1, 275)=26
 KSPARSE(2, 275)=5
!ONIT/OH
 KSPARSE(1, 276)=26
 KSPARSE(2, 276)=14
!ONIT/HO2
 KSPARSE(1, 277)=26
 KSPARSE(2, 277)=15
!ONIT/ONIT
 KSPARSE(1, 278)=26
 KSPARSE(2, 278)=26
!ONIT/PAN
 KSPARSE(1, 279)=26
 KSPARSE(2, 279)=27
!ONIT/MO2
 KSPARSE(1, 280)=26
 KSPARSE(2, 280)=31
!ONIT/ALKAP
 KSPARSE(1, 281)=26
 KSPARSE(2, 281)=32
!ONIT/BIOP
 KSPARSE(1, 282)=26
 KSPARSE(2, 282)=34
!ONIT/PHO
 KSPARSE(1, 283)=26
 KSPARSE(2, 283)=35
!ONIT/AROP
 KSPARSE(1, 284)=26
 KSPARSE(2, 284)=37
!ONIT/CARBOP
 KSPARSE(1, 285)=26
 KSPARSE(2, 285)=38
!ONIT/OLN
 KSPARSE(1, 286)=26
 KSPARSE(2, 286)=39
!PAN/O3
 KSPARSE(1, 287)=27
 KSPARSE(2, 287)=1
!PAN/NO2
 KSPARSE(1, 288)=27
 KSPARSE(2, 288)=4
!PAN/NO3
 KSPARSE(1, 289)=27
 KSPARSE(2, 289)=5
!PAN/OH
 KSPARSE(1, 290)=27
 KSPARSE(2, 290)=14
!PAN/PAN
 KSPARSE(1, 291)=27
 KSPARSE(2, 291)=27
!PAN/CARBOP
 KSPARSE(1, 292)=27
 KSPARSE(2, 292)=38
!OP1/OH
 KSPARSE(1, 293)=28
 KSPARSE(2, 293)=14
!OP1/HO2
 KSPARSE(1, 294)=28
 KSPARSE(2, 294)=15
!OP1/OP1
 KSPARSE(1, 295)=28
 KSPARSE(2, 295)=28
!OP1/MO2
 KSPARSE(1, 296)=28
 KSPARSE(2, 296)=31
!OP2/O3
 KSPARSE(1, 297)=29
 KSPARSE(2, 297)=1
!OP2/OH
 KSPARSE(1, 298)=29
 KSPARSE(2, 298)=14
!OP2/HO2
 KSPARSE(1, 299)=29
 KSPARSE(2, 299)=15
!OP2/CARBO
 KSPARSE(1, 300)=29
 KSPARSE(2, 300)=25
!OP2/OP2
 KSPARSE(1, 301)=29
 KSPARSE(2, 301)=29
!OP2/ALKAP
 KSPARSE(1, 302)=29
 KSPARSE(2, 302)=32
!OP2/ALKEP
 KSPARSE(1, 303)=29
 KSPARSE(2, 303)=33
!OP2/BIOP
 KSPARSE(1, 304)=29
 KSPARSE(2, 304)=34
!OP2/AROP
 KSPARSE(1, 305)=29
 KSPARSE(2, 305)=37
!OP2/CARBOP
 KSPARSE(1, 306)=29
 KSPARSE(2, 306)=38
!OP2/XO2
 KSPARSE(1, 307)=29
 KSPARSE(2, 307)=40
!ORA2/O3
 KSPARSE(1, 308)=30
 KSPARSE(2, 308)=1
!ORA2/HO2
 KSPARSE(1, 309)=30
 KSPARSE(2, 309)=15
!ORA2/ALKE
 KSPARSE(1, 310)=30
 KSPARSE(2, 310)=19
!ORA2/BIO
 KSPARSE(1, 311)=30
 KSPARSE(2, 311)=20
!ORA2/CARBO
 KSPARSE(1, 312)=30
 KSPARSE(2, 312)=25
!ORA2/MO2
 KSPARSE(1, 313)=30
 KSPARSE(2, 313)=31
!ORA2/ALKAP
 KSPARSE(1, 314)=30
 KSPARSE(2, 314)=32
!ORA2/ALKEP
 KSPARSE(1, 315)=30
 KSPARSE(2, 315)=33
!ORA2/BIOP
 KSPARSE(1, 316)=30
 KSPARSE(2, 316)=34
!ORA2/CARBOP
 KSPARSE(1, 317)=30
 KSPARSE(2, 317)=38
!ORA2/OLN
 KSPARSE(1, 318)=30
 KSPARSE(2, 318)=39
!MO2/O3
 KSPARSE(1, 319)=31
 KSPARSE(2, 319)=1
!MO2/NO
 KSPARSE(1, 320)=31
 KSPARSE(2, 320)=3
!MO2/NO3
 KSPARSE(1, 321)=31
 KSPARSE(2, 321)=5
!MO2/OH
 KSPARSE(1, 322)=31
 KSPARSE(2, 322)=14
!MO2/HO2
 KSPARSE(1, 323)=31
 KSPARSE(2, 323)=15
!MO2/CH4
 KSPARSE(1, 324)=31
 KSPARSE(2, 324)=16
!MO2/ALKE
 KSPARSE(1, 325)=31
 KSPARSE(2, 325)=19
!MO2/BIO
 KSPARSE(1, 326)=31
 KSPARSE(2, 326)=20
!MO2/ALD
 KSPARSE(1, 327)=31
 KSPARSE(2, 327)=23
!MO2/OP1
 KSPARSE(1, 328)=31
 KSPARSE(2, 328)=28
!MO2/OP2
 KSPARSE(1, 329)=31
 KSPARSE(2, 329)=29
!MO2/MO2
 KSPARSE(1, 330)=31
 KSPARSE(2, 330)=31
!MO2/ALKAP
 KSPARSE(1, 331)=31
 KSPARSE(2, 331)=32
!MO2/ALKEP
 KSPARSE(1, 332)=31
 KSPARSE(2, 332)=33
!MO2/BIOP
 KSPARSE(1, 333)=31
 KSPARSE(2, 333)=34
!MO2/AROP
 KSPARSE(1, 334)=31
 KSPARSE(2, 334)=37
!MO2/CARBOP
 KSPARSE(1, 335)=31
 KSPARSE(2, 335)=38
!MO2/OLN
 KSPARSE(1, 336)=31
 KSPARSE(2, 336)=39
!MO2/XO2
 KSPARSE(1, 337)=31
 KSPARSE(2, 337)=40
!ALKAP/O3
 KSPARSE(1, 338)=32
 KSPARSE(2, 338)=1
!ALKAP/NO
 KSPARSE(1, 339)=32
 KSPARSE(2, 339)=3
!ALKAP/NO3
 KSPARSE(1, 340)=32
 KSPARSE(2, 340)=5
!ALKAP/OH
 KSPARSE(1, 341)=32
 KSPARSE(2, 341)=14
!ALKAP/HO2
 KSPARSE(1, 342)=32
 KSPARSE(2, 342)=15
!ALKAP/ETH
 KSPARSE(1, 343)=32
 KSPARSE(2, 343)=17
!ALKAP/ALKA
 KSPARSE(1, 344)=32
 KSPARSE(2, 344)=18
!ALKAP/ALKE
 KSPARSE(1, 345)=32
 KSPARSE(2, 345)=19
!ALKAP/BIO
 KSPARSE(1, 346)=32
 KSPARSE(2, 346)=20
!ALKAP/KET
 KSPARSE(1, 347)=32
 KSPARSE(2, 347)=24
!ALKAP/ONIT
 KSPARSE(1, 348)=32
 KSPARSE(2, 348)=26
!ALKAP/OP2
 KSPARSE(1, 349)=32
 KSPARSE(2, 349)=29
!ALKAP/MO2
 KSPARSE(1, 350)=32
 KSPARSE(2, 350)=31
!ALKAP/ALKAP
 KSPARSE(1, 351)=32
 KSPARSE(2, 351)=32
!ALKAP/CARBOP
 KSPARSE(1, 352)=32
 KSPARSE(2, 352)=38
!ALKEP/NO
 KSPARSE(1, 353)=33
 KSPARSE(2, 353)=3
!ALKEP/NO3
 KSPARSE(1, 354)=33
 KSPARSE(2, 354)=5
!ALKEP/OH
 KSPARSE(1, 355)=33
 KSPARSE(2, 355)=14
!ALKEP/HO2
 KSPARSE(1, 356)=33
 KSPARSE(2, 356)=15
!ALKEP/ALKE
 KSPARSE(1, 357)=33
 KSPARSE(2, 357)=19
!ALKEP/MO2
 KSPARSE(1, 358)=33
 KSPARSE(2, 358)=31
!ALKEP/ALKEP
 KSPARSE(1, 359)=33
 KSPARSE(2, 359)=33
!ALKEP/CARBOP
 KSPARSE(1, 360)=33
 KSPARSE(2, 360)=38
!BIOP/NO
 KSPARSE(1, 361)=34
 KSPARSE(2, 361)=3
!BIOP/NO3
 KSPARSE(1, 362)=34
 KSPARSE(2, 362)=5
!BIOP/OH
 KSPARSE(1, 363)=34
 KSPARSE(2, 363)=14
!BIOP/HO2
 KSPARSE(1, 364)=34
 KSPARSE(2, 364)=15
!BIOP/ALKE
 KSPARSE(1, 365)=34
 KSPARSE(2, 365)=19
!BIOP/BIO
 KSPARSE(1, 366)=34
 KSPARSE(2, 366)=20
!BIOP/MO2
 KSPARSE(1, 367)=34
 KSPARSE(2, 367)=31
!BIOP/BIOP
 KSPARSE(1, 368)=34
 KSPARSE(2, 368)=34
!BIOP/CARBOP
 KSPARSE(1, 369)=34
 KSPARSE(2, 369)=38
!PHO/NO2
 KSPARSE(1, 370)=35
 KSPARSE(2, 370)=4
!PHO/NO3
 KSPARSE(1, 371)=35
 KSPARSE(2, 371)=5
!PHO/OH
 KSPARSE(1, 372)=35
 KSPARSE(2, 372)=14
!PHO/HO2
 KSPARSE(1, 373)=35
 KSPARSE(2, 373)=15
!PHO/ARO
 KSPARSE(1, 374)=35
 KSPARSE(2, 374)=21
!PHO/PHO
 KSPARSE(1, 375)=35
 KSPARSE(2, 375)=35
!ADD/O3
 KSPARSE(1, 376)=36
 KSPARSE(2, 376)=1
!ADD/NO2
 KSPARSE(1, 377)=36
 KSPARSE(2, 377)=4
!ADD/OH
 KSPARSE(1, 378)=36
 KSPARSE(2, 378)=14
!ADD/ARO
 KSPARSE(1, 379)=36
 KSPARSE(2, 379)=21
!ADD/ADD
 KSPARSE(1, 380)=36
 KSPARSE(2, 380)=36
!AROP/NO
 KSPARSE(1, 381)=37
 KSPARSE(2, 381)=3
!AROP/NO3
 KSPARSE(1, 382)=37
 KSPARSE(2, 382)=5
!AROP/HO2
 KSPARSE(1, 383)=37
 KSPARSE(2, 383)=15
!AROP/MO2
 KSPARSE(1, 384)=37
 KSPARSE(2, 384)=31
!AROP/ADD
 KSPARSE(1, 385)=37
 KSPARSE(2, 385)=36
!AROP/AROP
 KSPARSE(1, 386)=37
 KSPARSE(2, 386)=37
!AROP/CARBOP
 KSPARSE(1, 387)=37
 KSPARSE(2, 387)=38
!CARBOP/O3
 KSPARSE(1, 388)=38
 KSPARSE(2, 388)=1
!CARBOP/NO
 KSPARSE(1, 389)=38
 KSPARSE(2, 389)=3
!CARBOP/NO2
 KSPARSE(1, 390)=38
 KSPARSE(2, 390)=4
!CARBOP/NO3
 KSPARSE(1, 391)=38
 KSPARSE(2, 391)=5
!CARBOP/OH
 KSPARSE(1, 392)=38
 KSPARSE(2, 392)=14
!CARBOP/HO2
 KSPARSE(1, 393)=38
 KSPARSE(2, 393)=15
!CARBOP/ALKE
 KSPARSE(1, 394)=38
 KSPARSE(2, 394)=19
!CARBOP/BIO
 KSPARSE(1, 395)=38
 KSPARSE(2, 395)=20
!CARBOP/ALD
 KSPARSE(1, 396)=38
 KSPARSE(2, 396)=23
!CARBOP/KET
 KSPARSE(1, 397)=38
 KSPARSE(2, 397)=24
!CARBOP/CARBO
 KSPARSE(1, 398)=38
 KSPARSE(2, 398)=25
!CARBOP/PAN
 KSPARSE(1, 399)=38
 KSPARSE(2, 399)=27
!CARBOP/OP2
 KSPARSE(1, 400)=38
 KSPARSE(2, 400)=29
!CARBOP/MO2
 KSPARSE(1, 401)=38
 KSPARSE(2, 401)=31
!CARBOP/ALKAP
 KSPARSE(1, 402)=38
 KSPARSE(2, 402)=32
!CARBOP/ALKEP
 KSPARSE(1, 403)=38
 KSPARSE(2, 403)=33
!CARBOP/BIOP
 KSPARSE(1, 404)=38
 KSPARSE(2, 404)=34
!CARBOP/AROP
 KSPARSE(1, 405)=38
 KSPARSE(2, 405)=37
!CARBOP/CARBOP
 KSPARSE(1, 406)=38
 KSPARSE(2, 406)=38
!CARBOP/OLN
 KSPARSE(1, 407)=38
 KSPARSE(2, 407)=39
!CARBOP/XO2
 KSPARSE(1, 408)=38
 KSPARSE(2, 408)=40
!OLN/NO
 KSPARSE(1, 409)=39
 KSPARSE(2, 409)=3
!OLN/NO3
 KSPARSE(1, 410)=39
 KSPARSE(2, 410)=5
!OLN/HO2
 KSPARSE(1, 411)=39
 KSPARSE(2, 411)=15
!OLN/ALKE
 KSPARSE(1, 412)=39
 KSPARSE(2, 412)=19
!OLN/BIO
 KSPARSE(1, 413)=39
 KSPARSE(2, 413)=20
!OLN/CARBO
 KSPARSE(1, 414)=39
 KSPARSE(2, 414)=25
!OLN/MO2
 KSPARSE(1, 415)=39
 KSPARSE(2, 415)=31
!OLN/CARBOP
 KSPARSE(1, 416)=39
 KSPARSE(2, 416)=38
!OLN/OLN
 KSPARSE(1, 417)=39
 KSPARSE(2, 417)=39
!XO2/O3
 KSPARSE(1, 418)=40
 KSPARSE(2, 418)=1
!XO2/NO
 KSPARSE(1, 419)=40
 KSPARSE(2, 419)=3
!XO2/NO3
 KSPARSE(1, 420)=40
 KSPARSE(2, 420)=5
!XO2/OH
 KSPARSE(1, 421)=40
 KSPARSE(2, 421)=14
!XO2/HO2
 KSPARSE(1, 422)=40
 KSPARSE(2, 422)=15
!XO2/ALKE
 KSPARSE(1, 423)=40
 KSPARSE(2, 423)=19
!XO2/BIO
 KSPARSE(1, 424)=40
 KSPARSE(2, 424)=20
!XO2/ARO
 KSPARSE(1, 425)=40
 KSPARSE(2, 425)=21
!XO2/CARBO
 KSPARSE(1, 426)=40
 KSPARSE(2, 426)=25
!XO2/PAN
 KSPARSE(1, 427)=40
 KSPARSE(2, 427)=27
!XO2/OP2
 KSPARSE(1, 428)=40
 KSPARSE(2, 428)=29
!XO2/MO2
 KSPARSE(1, 429)=40
 KSPARSE(2, 429)=31
!XO2/ALKAP
 KSPARSE(1, 430)=40
 KSPARSE(2, 430)=32
!XO2/CARBOP
 KSPARSE(1, 431)=40
 KSPARSE(2, 431)=38
!XO2/XO2
 KSPARSE(1, 432)=40
 KSPARSE(2, 432)=40
KSPARSEDIM = 432
IF (LHOOK) CALL DR_HOOK('CH_SPARSE',1,ZHOOK_HANDLE)
RETURN
END SUBROUTINE CH_SPARSE
